/**
  ******************************************************************************
  * @file           : tid_api_inf.c
  * @brief          : tid api for user interface
  ******************************************************************************
  *
  * Copyright (c) 2019 Tencent. 
  * All rights reserved.
  ******************************************************************************
  */
#ifndef __TID_API_INF_H__
#define __TID_API_INF_H__
#include "tid_driver_comm.h"

#define TID_VAL_LEN                 (16)
#define TID_SEND_BUFF_LEN			(512)
#define TID_ECC_PUBKEY_LEN          (68)
#define TID_RETBUF_LEN_MIN          (32)
#define TID_SYM_KEY_LEN             (32)
#define TID_VENDOR_INFO_LEN         (64)
#define TID_ASYM_SIGN_LEN           (64)
#define TID_SYM_IV_LEN              (16)

#define TID_MAX_SEG_LEN             (0xc0)

typedef enum _eClientState_{
	eNOT_INITIALIZED = 0,
	eINITIALIZED  = 1,
}eClientState;

typedef enum _eApduCase_{
	eAPDU_CASE_A = 0,		//APDU Case A: CLA INS P1 P2 00
	eAPDU_CASE_B = 1,		//APDU Case B: CLA INS P1 P2 Le
	eAPDU_CASE_C = 2,		//APDU Case C: CLA INS P1 P2 Lc Data
	eAPDU_CASE_D = 3,		//APDU Case D: CLA INS P1 P2 Lc Data Le
}eApduCase;

enum {
 TID_CLA_COMMON  = 0x00,
 TID_CLA_EXT  = 0x80,
 TID_CLA_EXT_S = 0x84,
};

enum {
    TID_SYM_KTYPE_AES_CBC = 0x02,
    TID_SYM_KTYPE_AES_ECB = 0x03,
    TID_SYM_KTYPE_SM4_CBC = 0x10,
    TID_SYM_KTYPE_SM4_ECB = 0x11,
};

enum {
    TID_ASYM_ALGO_SM2        = 0x05,
    TID_ASYM_ALGO_ECSDA      = 0x06,
};

enum {
    TID_INS_GET_CHALLENGE      = 0x84,
    TID_INS_SYM_CRYPTO         = 0xF0,
    TID_INS_ASYM_CRYPTO        = 0xF2,
    TID_INS_GEN_KEYPAIR        = 0xF4,
    TID_INS_GEN_SHAREDKEY      = 0xF6,
    TID_INS_GEN_SESSIONKEY     = 0xF8,
    TID_INS_COMPUTE_DIG        = 0xFA,
    TID_INS_GET_TID            = 0xFC,
    TID_INS_GET_VENDOR         = 0xCA,
    TID_INS_SEC_STORE          = 0xE2,
    TID_INS_IMPORT_SESS        = 0x5A,
};

enum {
    TID_KTYPE_AES           = 0x01,
    TID_KTYPE_SM2           = 0x04,
    TID_KTYPE_SM4           = 0x05,
    TID_KTYPE_ECC256        = 0x0A,
    TID_KTYPE_TID           = 0x0B,
};

enum {
    TID_ECDH_ALGO_ECC256    = 0x01,
    TID_ECDH_ALGO_SM2       = 0x02,
};

enum {
    TID_DIGEST_SHA256       = 0x02,
    TID_DIGEST_SM3          = 0x03,
};

enum {
    TID_KID_TID             = 0xA0,
    TID_KID_PRIVKEY         = 0x80,
    TID_KID_PUBKEY          = 0x81,
    TID_KID_PSK             = 0x90,
    TID_KID_COMPINFO        = 0xA1,
    TID_KID_COMPPUBKEY      = 0x83,
    TID_KID_TEMPPRIVKEY     = 0xF0,
    TID_KID_TEMPPUBKEY      = 0xF1,
    TID_KID_SYMKEY0         = 0xF2,
    TID_KID_SYMKEY1,
    TID_KID_SYMKEY2,
    TID_KID_SYMKEY3,
    TID_KID_TEMPSHARED      = 0xFA,
    TID_KID_SESSKEY         = 0xFB,  
};

enum {
    TID_SW_SUCC     = 0x9000,
};

enum {
    TID_ASYM_TYPE_SIGN      = 0x53,
    TID_ASYM_TYPE_VERIFY    = 0x54,
};

typedef struct{
	eBusType 	 bus_type;
	eVendor  	 vendor;
	sTidFunc     *pfun;
	sApduPack    tid_cmd;
	eClientState state;
	eApduCase    cmd_case;
	uint8_t	     send_buf[TID_SEND_BUFF_LEN];
	uint16_t     send_len;
} sTidClient;


typedef struct {
    uint8_t     asym_type;
} sTidAsymOperation;


typedef struct {
    uint8_t     sym_type;
    uint8_t     sym_enc;
    uint8_t     kid;
    uint8_t     iv[16];
} sTidSymOperation;

typedef struct {
    uint8_t     digest_type;
    uint8_t     data_type;
    uint8_t     kid;
} sTidDigestOperation;


typedef struct {
    uint8_t     block_id;
    void        *operation;
} sTidCryptoContext;


/** 
 * @brief Get the handle of tid client operation
 * @return  the handle of tid client   
 */
void *get_tid_client_handle(void);

/** 
 * @brief Initialize the symmetrical cryption
 * @param    ctx                      context
 * @param    sym_type                 symmetrical cryption type
 * @param    enc_algo                 algorithm
 * @param    kid                      key index
 * @param    iv                       iv for CBC computation
 * @return                            operation result
 */
eTidResult tid_sym_crypto_init(sTidCryptoContext *ctx, uint8_t sym_type, uint8_t enc_algo, uint8_t kid, uint8_t *iv);


/** 
 * @brief Update information for sym cryption
 * @param    ctx                      context for sym cryption
 * @param    in_buf                   input buffer
 * @param    in_buf_len               the length of input buffer
 * @param    out_buf                  output buffer
 * @param    out_buf_len              the length of output buffer
 * @param    is_last                  is the last update or not
 * @return                            operation result
 */
eTidResult tid_sym_crypto_update(sTidCryptoContext *ctx, uint8_t *in_buf, int in_buf_len, uint8_t *out_buf, uint16_t *out_buf_len, int is_last);


/*
* @brief init tid client, should be run before call any tid function
*
* @return @see eTidResult

*/
void *tid_client_init(eBusType type, eVendor vendor);



/**
 * @brief deinit tid client
 *
 * @return @see eTidResult
 */
eTidResult tid_client_destroy(void);


/**
 * @brief  get the tid sdk version info
 * @param  pver  version info for output buff
 * @param  len   len of output buff can be used
 *
 * @return @see irot_result_t
 */
eTidResult tid_get_sdk_ver(char* pver, uint16_t len);


/**
 * @brief get vendor
 *
 * @param vendor   vendor buffer, not less than TID_VENDOR_INFO_LEN.
 * @param rlen  tid data len has been read back
 *
 * @return @see eTidResult
 */

eTidResult tid_get_vendor(uint8_t *vendor, uint16_t *rlen);

/**
 * @brief get tid 
 *
 * @param tid   tid buffer, not less than TID_VAL_LEN.
 * @param rlen  tid data len has been read back
 *
 * @return @see eTidResult
 */

eTidResult tid_get_tid(uint8_t *tid, uint16_t *rlen);



/**
 * @brief get the random number form se. 
 * @param destlen   expect random len
 * @param rand_buf   buff for random output
 * @param rand_buflen length of Random value
 *
 * @return @see eTidResult
 */

eTidResult tid_get_challenge(uint16_t destlen, uint8_t *rand_buf, uint16_t *rand_buflen);

/** 
 * @brief    Import Session Key to SE
 * @param    in_buf                   key buffer
 * @param    in_buf_len               key length
 * @param    key_type                 symmetrical key type AES or SM4
 * @param    kid                      key index
 * @return                            Operation result
 */
eTidResult tid_import_sesskey(const uint8_t *in_buf, uint16_t in_buf_len, int key_type, uint8_t kid);


/** 
 * @brief    Symmetrical encryption
 * @param    enc_type                 key type for encryption
 * @param    in_buf                   input buffer
 * @param    in_buf_len               buffer length
 * @param    out_buf                  output buffer
 * @param    out_buf_len              output buffer length pointer
 * @param    kid                      the key index
 * @return                            Operation result
 */
eTidResult tid_sym_enc(int enc_type, const uint8_t *in_buf, int in_buf_len, 
                        uint8_t *out_buf, uint16_t *out_buf_len, uint8_t kid);


/** 
 * @brief    Symmetrical decryption
 * @param    enc_type                 key type for encryption
 * @param    in_buf                   input buffer
 * @param    in_buf_len               buffer length
 * @param    out_buf                  output buffer
 * @param    out_buf_len              output buffer length pointer
 * @param    kid                      the key index
 * @return                            Operation result
 */
eTidResult tid_sym_dec(uint8_t enc_type, const uint8_t *in_buf, int in_buf_len, 
                        uint8_t *out_buf, uint16_t *out_buf_len, uint8_t kid);

/** 
 * @brief    asymmetrical sign
 * @param    in_buf                   input buffer
 * @param    in_buf_len               length of input buffer
 * @param    out_buf                  buffer for the signature
 * @param    out_buf_len              the length of the signature
 * @param    kid                      private key index for sign
 * @param    algo                     asymmetrical algorithm
 * @return                            operation result
 */
eTidResult tid_asym_crypto_sign(const uint8_t *in_buf, int in_buf_len, uint8_t *out_buf, uint16_t *out_buf_len, uint8_t kid, uint8_t algo);


eTidResult tid_asym_crypto_verify_inner(const uint8_t *in_buf, int in_buf_len, const uint8_t *sign_buf, uint8_t kid, uint8_t algo);
eTidResult tid_asym_crypto_verify_outer(const uint8_t *in_buf, int in_buf_len, const uint8_t *sign_buf, const uint8_t *pbkey, uint8_t algo);


/** 
 * @brief    Initialize asym cryption
 * @param    ctx                      Context to store asym operation
 * @param    asym_type                Asym type
 * @param    asym_algo                Asym algorithm
 * @param    key_source               Key source: inner or outer
 * @param    kid                      [description]
 * @param    key_buf                  [description]
 * @return                            [description]
 */
eTidResult tid_asym_crypto_init(sTidCryptoContext *ctx, uint8_t asym_type, uint8_t asym_algo, uint8_t key_source, uint8_t kid, uint8_t *key_buf);


/** 
 * @brief [tid_asym_crypto_update description]
 * @param    ctx                      [description]
 * @param    in_buf                   [description]
 * @param    in_buf_len               [description]
 * @param    out_buf                  [description]
 * @param    out_buf_len              [description]
 * @param    is_last                  [description]
 * @param    sign                     [description]
 * @return                            [description]
 */
eTidResult tid_asym_crypto_update(sTidCryptoContext *ctx, uint8_t *in_buf, int in_buf_len, uint8_t *out_buf, uint16_t *out_buf_len, int is_last, uint8_t *sign);


/** 
 * @brief [tid_compute_digest description]
 * @param    kid                      [description]
 * @param    in_buf                   [description]
 * @param    in_buf_len               [description]
 * @param    out_buf                  [description]
 * @param    out_buf_len              [description]
 * @param    algo                     [description]
 * @return                            [description]
 */
eTidResult tid_compute_digest(int kid, const uint8_t *in_buf, int in_buf_len, uint8_t *out_buf, uint16_t *out_buf_len, int algo);


/** 
 * @brief   Generate temporary key pair
 * @param    pub_key                  The public key of the key pair
 * @param    out_pk_len               The length of public key
 * @param    algo                     The algorithm of the key pair
 * @return                            operation result
 */
eTidResult tid_gen_temp_keypair(uint8_t *pub_key, uint16_t *out_pk_len, uint8_t algo);

/** 
 * @brief [tid_gen_token description]
 * @param    in_pubkey                [description]
 * @param    in_pubkey_len            [description]
 * @param    algo                     [description]
 * @param    no_return                [description]
 * @param    out_buf                  [description]
 * @return                            [description]
 */
eTidResult tid_gen_token(const uint8_t *in_pubkey, uint16_t in_pubkey_len, uint8_t algo, 
                    int no_return, uint8_t *out_buf);


/**
 * @brief tid generate a shared key
 *
 * @param wait bo be define
 *
 * @return @see eTidResult
 */

eTidResult tid_gen_session_key(const uint8_t *prf, uint16_t prf_len);

eTidResult tid_compute_digest(int kid, const uint8_t *in_buf, int in_buf_len, uint8_t *out_buf, uint16_t *out_buf_len, int algo);

eTidResult tid_get_dev_pubkey(uint8_t *out_data, uint16_t *out_data_len);

eTidResult tid_raw_apdu(uint8_t *apdu, uint16_t len, uint8_t *output, uint16_t *outlen);

#endif
