/*
 * Tencent is pleased to support the open source community by making IoT Hub available.
 * Copyright (C) 2019 THL A29 Limited, a Tencent company. All rights reserved.

 * Licensed under the MIT License (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://opensource.org/licenses/MIT

 * Unless required by applicable law or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
#ifndef __TID_AUTH_H_
#define __TID_AUTH_H_
#include "tid_driver_comm.h"

#ifdef __cplusplus
extern "C"{
#endif

#define TID_PROTO_VERSION           0x01


enum {
    TID_SUITE_SECP256R1_FDM     = 0x01,
    TID_SUITE_SM2_SM4           = 0x10,
    TID_SUITE_SM2_AES128        = 0x11,
    TID_SUITE_SM2_AES256        = 0x12,
    TID_SUITE_SECP256R1_SM4     = 0x13,
    TID_SUITE_SECP256R1_AES128  = 0x14,
    TID_SUITE_SECP256R1_AES256  = 0x15,
};

/** 
 * @brief  Init the context of TID
 * @param    type                     The type of hardware interface, such as I2C/SPI
 * @param    vendor                   The SE IC chip vendor
 * @param    proto                    The proto version
 * @param    is_sm                    Use SM or not
 * @return                            The operation result
 */
eTidResult tid_client_se_init(eBusType type, eVendor vendor, int proto, int is_sm);

/** 
 * @brief   Get the TID client authcode from SE
 * @param    auth_buf                 The buffer used for output client authcode, should be no less than 132
 * @param    buf_len                  To indicate the length of auth_buf and return the actual length
 * @return                            The operation result
 */
eTidResult tid_client_get_authcode(uint8_t *auth_buf, int *buf_len);

/** 
 * @brief    Validate the authcode from the Server 
 * @param    auth_buf                 The buffer of the authcode from the server
 * @param    buf_len                  The length of the buffer
 * @param    token                    The output token
 * @param    rlen                     The retuen length of token
 * @return                            The operation result
 */
eTidResult tid_client_validate_authcode(uint8_t *auth_buf, int buf_len, uint8_t *token, uint16_t *rlen);

/** 
 * @brief  Generate session key within SE, please see TID Spec
 * @param    prf                      The random buffer
 * @param    prf_len                  The length of the random buffer
 * @return                            Operation result
 */
eTidResult tid_client_generate_sesskey(uint8_t *prf, int prf_len);


/** 
 * @brief  Import session key into the SE
 * @param    sess_key                 The buffer of session key
 * @param    key_len                  The length of session key
 * @param    key_index                The index of the session, start from 0
 * @return                            Operation result
 */
eTidResult tid_client_import_sesskey(uint8_t *sess_key, int key_len, int key_index);


/** 
 * @brief Symmetrical cryption in SE, the input buffer should be properly padded
 * @param    in_buf                   Input buffer
 * @param    out_buf                  Output buffer
 * @param    in_buf_len               The length of input buffer
 * @param    key_index                The index of session, indicated in import
 * @param    is_enc                   Encrypt or decrypt
 * @return                            Operation result
 */
eTidResult tid_client_sym_crypto(const uint8_t *in_buf, uint8_t *out_buf, int in_buf_len, int key_index, int is_enc);


/** 
 * @brief  Calculate the digest of a message
 * @param    in_buf                   The input buffer to digest
 * @param    in_buf_len               The length of input buffer
 * @param    out_buf                  The output buffer of digestion
 * @return                            Operation result
 */
eTidResult tid_client_digest(const uint8_t *in_buf, int in_buf_len, uint8_t *out_buf);

/** 
 * @brief  Calculate the signature of message with private key in SE
 * @param    in_buf                   The input buffer
 * @param    in_buf_len               The length of input buffer
 * @param    out_buf                  The output buffer for signature
 * @param    out_buf_len              The length of signaure
 * @return                            Operation
 */
eTidResult tid_client_asym_sign(const uint8_t *in_buf, int in_buf_len, uint8_t *out_buf, int *out_buf_len);

/** 
 * @brief  Verify the signaure of message with public key, the algorithm is chosen in init
 * @param    in_msg                   The input message
 * @param    in_msg_len               The length of input message
 * @param    in_pubkey                The public key to verify message, length is based on the algorithm
 * @param    in_sign_buf              The signature buffer, length is based on the algorithm
 * @return                            Operation result
 */
eTidResult tid_client_asym_verify(const uint8_t *in_msg, int in_msg_len, const uint8_t *in_pubkey, const uint8_t *in_sign_buf);

#ifdef __cplusplus
}
#endif
#endif
