#include "tid_export.h"

#if (TID_BUS_TYPE == TID_BUS_I2C)
#include "stm32f10x.h"
#include "bitband.h"
#include "port_i2c.h"


/********************************************************************
* Function: I2C SE power on
* Input Parameter:None
* Output Parameter:None
* Return: None
*********************************************************************/
void user_i2c_dev_power_on(void)
{
	//SE_VCC_EN--PE4,SE_IRQ--PD9
	GPIOE->CRL &= 0xFFF0FFFF;
	GPIOE->CRL |= 0x00030000;
	
	//power on
	PEout(4) = 1;
	HAL_DelayMs(I2C_PWR_ON_DELAY);
}

/********************************************************************
* Function: I2C SE power off
* Input Parameter:None
* Output Parameter:None
* Return: None
*********************************************************************/
void user_i2c_dev_power_off(void)
{
	//SE_VCC_EN--PE4,SE_IRQ--PD9
	GPIOE->CRL &= 0xFFF0FFFF;
	GPIOE->CRL |= 0x00030000;
	
	//power off
	PEout(4) = 0;
	HAL_DelayMs(I2C_PWR_OFF_DELAY);
}

/********************************************************************
* Function: I2C SE interface and IO init
* Input Parameter:None
* Output Parameter:None
* Return: None
*********************************************************************/
void user_i2c_init(void)
{
	//enable I2C2 clk
	RCC->APB1ENR |= 1<<22;
	
	//hard_i2c io init
	GPIOB->CRH &= 0xFFFF00FF;
	GPIOB->CRH |= 0x0000EE00;

	//i2c reg init
	I2C2->CR1 |= 1<<15;
	I2C2->CR1 &= ~(1<<15);
	I2C2->CR2 |= 36;
	//100Kbps
	I2C2->CCR |= 180;
	I2C2->TRISE = 37;
	I2C2->OAR1 |= 1<<14;
	
	//enable ack,enable i2c module
	I2C2->CR1 |= 1<<10 | 1;
	
	HAL_DelayMs(200);
}

/********************************************************************
* Function: generate a I2C start signal
* Input Parameter:None
* Output Parameter:None
* Return: None
*********************************************************************/
void user_i2c_start(void)
{
	uint16_t count;

	//enable ack
	I2C2->CR1 |= 1<<10;
	//start
	I2C2->CR1 |= 1<<8;
	
	count = CHECK_TIMES;
	do{
		count--;
		if((I2C2->SR1&1)&&(I2C2->SR2&3))
		{
			break;
		}
	}while(count);
}

/********************************************************************
* Function: generate a I2C stop signal
* Input Parameter:None
* Output Parameter:None
* Return: None
*********************************************************************/
void user_i2c_stop(void)
{
	//disable ack
	I2C2->CR1 &= ~(1<<10);
	//stop
	I2C2->CR1 |= 1<<9;
}

/********************************************************************
* Function: Master send a I2C Byte to slaver,and receive slaver ACK
* Input Parameter:
* ch: the send byte
* Output Parameter:None
* Return: result
* 0-success
* 12-wait ack timeout
*********************************************************************/
eTidResult user_i2c_send_char(uint8_t ch)
{
	uint16_t count;
	Timer timer;
	eTidResult ret = TID_ERR_RECV_ACK;

	I2C2->DR = ch;
	
	HAL_Timer_countdown_ms(&timer, WAIT_TIME_MS);
	do{
		if((I2C2->SR1&0x84)&&(I2C2->SR2&7))
		{
			ret = TID_ERR_SUCCESS;
			break;
		}
	}while(!HAL_Timer_expired(&timer));	

	return ret;
}

/********************************************************************
* Function: Master receive a I2C Byte from slaver
* Input Parameter:None
* Output Parameter:None
* Return: master received byte from slaver
*********************************************************************/
uint8_t user_i2c_recv_char(void)
{	
	Timer timer;

	HAL_Timer_countdown_ms(&timer, WAIT_TIME_MS);
	do{
		if((I2C2->SR1&0x40)&&(I2C2->SR2&3))
		{
			break;
		}
	  }while(!HAL_Timer_expired(&timer));	


	if(HAL_Timer_expired(&timer))
	{
		Log_e("recv char time out");
	}
        
    return I2C2->DR;
}

/********************************************************************
* Function: Master send a I2C Addr to slaver,and receive slaver ACK
* Input Parameter:
* ch: the send byte
* Output Parameter:None
* Return: result
* 0-success
* 12-wait ack timeout
*********************************************************************/
eTidResult user_i2c_send_addr(uint8_t ch)
{
	Timer timer;
	eTidResult ret = TID_ERR_RECV_ACK;

	I2C2->DR = ch;
	HAL_Timer_countdown_ms(&timer, WAIT_TIME_MS);
	do{
		if((I2C2->SR1&2)&&(I2C2->SR2&3))
		{
			ret = TID_ERR_SUCCESS;
			break;
		}
	}while(!HAL_Timer_expired(&timer));

	return ret;
}

/********************************************************************
* Function: I2C Master Receive DATA(start->send addr+R->receivedata->stop)
* Input Parameter: *rbuf: receive data buf
* Output Parameter: rlen: receive len
* Return: None
*********************************************************************/

eTidResult user_i2c_recv_buf(uint8_t *rbuf,uint16_t rlen)
{
	return TID_ERR_NOT_SUPP;
}
/********************************************************************
* Function: I2C Master Send DATA(start->send addr+W->send data->stop)
* Input Parameter: *sbuf: send data buf
* Output Parameter: slen: send len
* Return: None
*********************************************************************/
eTidResult user_i2c_send_buf(uint8_t *sbuf,uint16_t slen)
{
    return TID_ERR_NOT_SUPP;
}

void user_i2c_dev_wake(void)
{

}

void user_i2c_dev_standby(void)
{

}


//global variable of I2C interface driver
sTid_I2C_Driver gusr_i2c_drv = 
{
	SE_ADDR,
	user_i2c_dev_power_on,
	user_i2c_dev_power_off,
	user_i2c_init,
	user_i2c_start,
	user_i2c_stop,
	user_i2c_send_char,
	user_i2c_recv_char,
	user_i2c_send_addr,
	user_i2c_recv_buf,
	user_i2c_send_buf,
	user_i2c_dev_wake,
	user_i2c_dev_standby
};


#endif
