/**
  ******************************************************************************
  * Copyright (c) 2019 Tencent. 
  * All rights reserved.
  ******************************************************************************

  ******************************************************************************
  * @file           : tid_api_inf.c
  * @brief          : tid api for user interface
  ******************************************************************************
*/

#include "tid_api_inf.h"
#include "tid_sanity_check.h"

#include <string.h>     /* memcpy */

static sTidClient g_tidClient = {0};
static sTidAsymOperation    _asym_operation;
static sTidSymOperation     _sym_operation;
static sTidDigestOperation  _digest_operation;

/*
* @brief init tid client, should be run before call any tid function
*
* @return @see eTidResult

*/
void *tid_client_init(eBusType type, eVendor vendor)
{	
	g_tidClient.pfun = tid_se_register(type, vendor);

	if(NULL != g_tidClient.pfun)
	{
		g_tidClient.bus_type = type;
		g_tidClient.vendor = vendor;
		g_tidClient.state = eINITIALIZED;
	}
	
	return (void *)&g_tidClient;
}

/*
* @brief get tid client handle, NULL will be returned if not inited success before
*
* @return @see eTidResult
*/

void *get_tid_client_handle(void)
{	
	void *pClient = NULL;
	
	if(eINITIALIZED == g_tidClient.state)
	{
		pClient = (void *)&g_tidClient;
	}

	return pClient;
}


/**
 * @brief deinit tid client
 *
 * @return @see eTidResult
 */
eTidResult tid_client_destroy(void)
{
	sTidClient *pClient = (sTidClient *)get_tid_client_handle();

	tid_se_unregister(pClient->pfun);	
	memset(&(pClient->tid_cmd), 0, APDU_DATA_MAX_LEN);
	pClient->state = eNOT_INITIALIZED;
	
	return TID_ERR_SUCCESS;
}

/**
 * @brief  get the tid sdk version info
 * @param  pver  version info for output buff
 * @param  len   len of output buff can be used
 *
 * @return @see irot_result_t
 */
eTidResult tid_get_sdk_ver(char* pver, uint16_t len)
{
	eTidResult ret = TID_ERR_SUCCESS;

	if((NULL == pver)||(len < strlen(TID_SDK_VERSION)))
	{
		return TID_ERR_INVAL;
	}
	
	strncpy(pver,TID_SDK_VERSION, len);
	
	return ret;
}

/**
 * @brief serialize tid apdu packet
 *
 * @param pClient   pointer of tid client
 *
 * @return @see  length of data has been serialized
 */
static uint16_t _serialize_tid_apdu_packet(sTidClient *pClient)
{
	POINTER_SANITY_CHECK(pClient, 0); 
	
	memset(pClient->send_buf, 0, TID_SEND_BUFF_LEN);
	pClient->send_len = 0;
	pClient->send_buf[pClient->send_len++] = pClient->tid_cmd.cla;
	pClient->send_buf[pClient->send_len++] = pClient->tid_cmd.ins;
	pClient->send_buf[pClient->send_len++] = pClient->tid_cmd.p1;
	pClient->send_buf[pClient->send_len++] = pClient->tid_cmd.p2;

	switch (pClient->cmd_case)
	{
		case eAPDU_CASE_A:
			 pClient->send_buf[pClient->send_len++] = 0;
			 break;
			 
		case eAPDU_CASE_B:
			 pClient->send_buf[pClient->send_len++] = pClient->tid_cmd.le;
			 break;	

		case eAPDU_CASE_C:
			 pClient->send_buf[pClient->send_len++] = pClient->tid_cmd.lc;
			 if((pClient->send_len + pClient->tid_cmd.data_len) < TID_SEND_BUFF_LEN)
			 {
				memcpy(&(pClient->send_buf[pClient->send_len]), pClient->tid_cmd.apdu, pClient->tid_cmd.data_len);
			 	pClient->send_len += pClient->tid_cmd.data_len;
			 }
			 else
			 {
			 	Log_e("case C apdu data len exceed max send buff len");
				pClient->send_len = 0;
			 }
			 break;	
			 
		case eAPDU_CASE_D:
			 pClient->send_buf[pClient->send_len++] = pClient->tid_cmd.lc;
			 if((pClient->send_len + pClient->tid_cmd.data_len) < TID_SEND_BUFF_LEN)
			 {
				memcpy(&(pClient->send_buf[pClient->send_len]), pClient->tid_cmd.apdu, pClient->tid_cmd.data_len);
			 	pClient->send_len += pClient->tid_cmd.data_len;
				pClient->send_buf[pClient->send_len++] = pClient->tid_cmd.le;
			 }
			 else
			 {
			 	Log_e("case D apdu data len exceed max send buff len");
				pClient->send_len = 0;
			 }
			 break;

		default:
			 Log_e("illegal apdu cmd case, %d", pClient->cmd_case);
			 pClient->send_len = 0;
	}

	return pClient->send_len;
}


/**
 * @brief get vendor
 *
 * @param vendor   vendor buffer, not less than TID_VENDOR_INFO_LEN.
 * @param rlen  tid data len has been read back
 *
 * @return @see eTidResult
 */

eTidResult tid_get_vendor(uint8_t *vendor, uint16_t *rlen)
{
	eTidResult ret;
	sTidClient *pClient;
	uint16_t sw;
	uint16_t interval = POLL_INTERVAL;
	uint32_t timeout = POLL_TIMEOUT;
    uint8_t tmp_data[128];
	
	pClient = (sTidClient *)get_tid_client_handle();
    if(NULL == pClient)
    {
        Log_e("Tid client is null , pls init tid client first");
        return TID_ERR_FAILURE;        
    }

	
	pClient->cmd_case = eAPDU_CASE_B;
	pClient->tid_cmd.cla = TID_CLA_EXT;
	pClient->tid_cmd.ins = TID_INS_GET_VENDOR;
	pClient->tid_cmd.p1 = 0x0;
	pClient->tid_cmd.p2 = 0x0;
	pClient->tid_cmd.le = TID_VENDOR_INFO_LEN;
	
	if(_serialize_tid_apdu_packet(pClient) > 0)
	{
		ret = pClient->pfun->tid_apdu_transceive(pClient->send_buf, pClient->send_len, tmp_data,\
																 rlen, interval, timeout);
		if(TID_ERR_SUCCESS == ret)
		{
			sw = tmp_data[*rlen-2]<<8 | tmp_data[*rlen-1];		
#ifdef TID_DEBUG_EABLE		
			Log_d("sw:0x%02x", sw);
#endif		
            if (TID_SW_SUCC != sw) {
                Log_e("Got wrong APDU answer!!");
                ret = TID_ERR_APDU;
            } else {
                *rlen -= 2;
                memcpy(vendor, tmp_data, *rlen);
            }
		}
		else
		{
			Log_e("Failed to transcieive, ret:%d!!", ret);
		}
	}
	else
	{
		Log_e("Failed to serialize tid apdu packet!!");
		ret = TID_ERR_FAILURE;	
	}
		
	return ret;
}

/**
 * @brief get tid 
 *
 * @param tid   tid buffer, not less than TID_VAL_LEN.
 * @param rlen  tid data len has been read back
 *
 * @return @see eTidResult
 */

eTidResult tid_get_tid(uint8_t *tid, uint16_t *rlen)
{
	eTidResult ret;
	sTidClient *pClient;
	uint16_t sw;
	uint16_t interval = POLL_INTERVAL;
	uint32_t timeout = POLL_TIMEOUT;
    uint8_t tmp_data[128];
	
	pClient = (sTidClient *)get_tid_client_handle();
    if(NULL == pClient)
    {
        Log_e("Tid client is null , pls init tid client first");
        return TID_ERR_FAILURE;        
    }

	
	pClient->cmd_case = eAPDU_CASE_B;
	pClient->tid_cmd.cla = TID_CLA_EXT;
	pClient->tid_cmd.ins = TID_INS_GET_TID;
	pClient->tid_cmd.p1 = 0x0;
	pClient->tid_cmd.p2 = 0x0;
	pClient->tid_cmd.le = TID_VAL_LEN;
	
	if (_serialize_tid_apdu_packet(pClient) > 0)
	{
		ret = pClient->pfun->tid_apdu_transceive(pClient->send_buf, pClient->send_len, tmp_data,\
																 rlen, interval, timeout);
        if(TID_ERR_SUCCESS == ret)
        {
            sw = tmp_data[*rlen-2]<<8 | tmp_data[*rlen-1];      
#ifdef TID_DEBUG_EABLE		
            Log_d("sw:0x%02x", sw);
#endif		
            if (TID_SW_SUCC != sw) {
                Log_e("Got wrong APDU answer!!");
                ret = TID_ERR_APDU;
            } else {
                *rlen -= 2;
                memcpy(tid, tmp_data, *rlen);
            }
            
        }
        else
        {
            Log_e("Failed to transcieive, ret:%d!!", ret);
        }
    }
    else
    {
        Log_e("Failed to serialize tid apdu packet!!");
        ret = TID_ERR_FAILURE;  
    }


	return ret;
}



/**
 * @brief get the random number form se. 
 * @param destlen   expect random len
 * @param rand_buf   buff for random output
 * @param rand_buflen length of Random value
 *
 * @return @see eTidResult
 */

eTidResult tid_get_challenge(uint16_t destlen, uint8_t *rand_buf, uint16_t *rand_buflen)
{	
	eTidResult ret;
	sTidClient *pClient;
	uint16_t sw;
	uint16_t interval = POLL_INTERVAL;
	uint32_t timeout = POLL_TIMEOUT;
    uint8_t tmp_data[64];
    
	pClient = (sTidClient *)get_tid_client_handle();
    if(NULL == pClient)
    {
        Log_e("Tid client is null , pls init tid client first");
        return TID_ERR_FAILURE;        
    }

	
	pClient->cmd_case = eAPDU_CASE_B;
	pClient->tid_cmd.cla = TID_CLA_COMMON;
	pClient->tid_cmd.ins = TID_INS_GET_CHALLENGE;
	pClient->tid_cmd.p1 = 0x0;
	pClient->tid_cmd.p2 = 0x0;
	pClient->tid_cmd.le = destlen;
	
	if (_serialize_tid_apdu_packet(pClient) > 0)
	{
		ret = pClient->pfun->tid_apdu_transceive(pClient->send_buf, pClient->send_len, tmp_data,\
																 rand_buflen, interval, timeout);
        if(TID_ERR_SUCCESS == ret)
        {
            sw = tmp_data[*rand_buflen-2]<<8 | tmp_data[*rand_buflen-1];      
#ifdef TID_DEBUG_EABLE		
            Log_d("sw:0x%02x", sw);
#endif		
            if (TID_SW_SUCC != sw) {
                Log_e("Got wrong APDU answer!!");
                ret = TID_ERR_APDU;
            } else {
                *rand_buflen -= 2;
                memcpy(rand_buf, tmp_data, *rand_buflen);
            }
        }
        else
        {
            Log_e("Failed to transcieive, ret:%d!!", ret);
        }
    }
    else
    {
        Log_e("Failed to serialize tid apdu packet!!");
        ret = TID_ERR_FAILURE;  
    }
		
	return ret;
}

/** 
 * @brief    Import Session Key to SE
 * @param    in_buf                   key buffer
 * @param    in_buf_len               key length
 * @param    key_type                 symmetrical key type AES or SM4
 * @param    kid                      key index
 * @return                            Operation result
 */
eTidResult tid_import_sesskey(const uint8_t *in_buf, uint16_t in_buf_len, int key_type, uint8_t kid)
{
	eTidResult ret;
	sTidClient *pClient;
	uint16_t sw;
	uint16_t interval = POLL_INTERVAL;
	uint32_t timeout = POLL_TIMEOUT;
	uint8_t out_buf[32];
	uint16_t out_buf_len;
    uint16_t expect_key_len;

    pClient = (sTidClient *)get_tid_client_handle();
    if(NULL == pClient)
    {
        Log_e("Tid client is null , pls init tid client first");
        return TID_ERR_FAILURE;
    }	
        

    pClient->cmd_case = eAPDU_CASE_C;
    pClient->tid_cmd.cla = TID_CLA_EXT;
    pClient->tid_cmd.ins = TID_INS_IMPORT_SESS;
    pClient->tid_cmd.p1 = kid;
    pClient->tid_cmd.p2 = key_type;
    pClient->tid_cmd.lc = in_buf_len; 
    memcpy(&(pClient->tid_cmd.apdu[0]), in_buf, in_buf_len);
    pClient->tid_cmd.data_len = in_buf_len;

	if(_serialize_tid_apdu_packet(pClient) > 0)
	{
		ret = pClient->pfun->tid_apdu_transceive(pClient->send_buf, pClient->send_len, out_buf,\
																 &out_buf_len, interval, timeout);
        if(TID_ERR_SUCCESS == ret)
        {
            sw = out_buf[out_buf_len-2]<<8 | out_buf[out_buf_len-1];      
#ifdef TID_DEBUG_EABLE		
            Log_d("sw:0x%02x", sw);
#endif		
            if (TID_SW_SUCC != sw) {
                Log_e("Got wrong APDU answer!!");
                ret = TID_ERR_APDU;
            }
        }
        else
        {
            Log_e("Failed to transcieive, ret:%d!!", ret);
        }
    }
    else
    {
        Log_e("Failed to serialize tid apdu packet!!");
        ret = TID_ERR_FAILURE;  
    }

	return ret;
}


static eTidResult _tid_sym_encdec(int is_enc, const uint8_t *in_buf, int in_buf_len, 
                                uint8_t *out_buf, uint16_t *out_buf_len, 
                                uint8_t key_type, uint8_t kid)
{
    eTidResult ret;
    sTidClient *pClient;
    uint16_t sw;
    uint16_t interval = POLL_INTERVAL;
    uint32_t timeout = POLL_TIMEOUT;
    uint8_t  tmp_data[TID_MAX_SEG_LEN + 18];
    int data_index = 0;
    int blk_num = 0;
    int seg_len;
    int apdu_idx = 0;
    uint16_t tmp_out_len;

    pClient = (sTidClient *)get_tid_client_handle();
    if(NULL == pClient)
    {
        Log_e("Tid client is null , pls init tid client first");
        return TID_ERR_FAILURE;
    }

    pClient->cmd_case = eAPDU_CASE_D;
    pClient->tid_cmd.cla = TID_CLA_EXT;
    pClient->tid_cmd.ins = TID_INS_SYM_CRYPTO;
    pClient->tid_cmd.p2 = is_enc ? 0x51 : 0x52;
    *out_buf_len = 0;
    
    while (in_buf_len > 0) {
        pClient->tid_cmd.p1 = blk_num;
        apdu_idx = 0;

        if (in_buf_len <= TID_MAX_SEG_LEN) {
            pClient->tid_cmd.p1 |= 0x80;
            seg_len = in_buf_len;
        } else {
            seg_len = TID_MAX_SEG_LEN;
        }
        if (blk_num == 0) {
            pClient->tid_cmd.apdu[apdu_idx++] = key_type;
            pClient->tid_cmd.apdu[apdu_idx++] = kid;
            memset(&(pClient->tid_cmd.apdu[apdu_idx]), 0, TID_SYM_IV_LEN);
            apdu_idx += TID_SYM_IV_LEN;
            memcpy(&(pClient->tid_cmd.apdu[apdu_idx]), &in_buf[data_index], seg_len);
            apdu_idx += seg_len;
            pClient->tid_cmd.lc = apdu_idx;
        } else {
            memcpy(pClient->tid_cmd.apdu, &in_buf[data_index], seg_len);
            pClient->tid_cmd.lc = seg_len;
        }
        pClient->tid_cmd.data_len = pClient->tid_cmd.lc;
        pClient->tid_cmd.le = seg_len;
        
        if(_serialize_tid_apdu_packet(pClient) > 0)
        {
            ret = pClient->pfun->tid_apdu_transceive(pClient->send_buf, pClient->send_len, tmp_data,\
                                                                     &tmp_out_len, interval, timeout);
            if(TID_ERR_SUCCESS == ret)
            {
                sw = tmp_data[tmp_out_len - 2]<<8 | tmp_data[tmp_out_len - 1];
#ifdef TID_DEBUG_EABLE		
                Log_d("sw:0x%02x", sw);                 
#endif
                Log_d("tmp_out_len %d seg_len %d", tmp_out_len, seg_len);
                if (TID_SW_SUCC == sw && (tmp_out_len == seg_len + 2)) {
                    memcpy(&out_buf[data_index], tmp_data, tmp_out_len - 2);
                    *out_buf_len += (tmp_out_len - 2);
                } else {
                    Log_e("SymCrypto Error");
                    ret = TID_ERR_FAILURE;
                    break;
                }
            }
            else
            {
                Log_e("SymCrypto %d failed,ret:%d", is_enc, ret);
                break;
            }
        }
        else
        {
            Log_e("serialize tid apdu packet failed");
            ret = TID_ERR_FAILURE;  
            break;
        }
        data_index += seg_len;
        blk_num++;
        in_buf_len -= seg_len;
    } 

    return ret;
}


/** 
 * @brief    Symmetrical encryption
 * @param    enc_type                 key type for encryption
 * @param    in_buf                   input buffer
 * @param    in_buf_len               buffer length
 * @param    out_buf                  output buffer
 * @param    out_buf_len              output buffer length pointer
 * @param    kid                      the key index
 * @return                            Operation result
 */
eTidResult tid_sym_enc(int enc_type, const uint8_t *in_buf, int in_buf_len, 
                        uint8_t *out_buf, uint16_t *out_buf_len, uint8_t kid)
{
    if (0 != (in_buf_len & 0xf)) {
        Log_e("The length of input data must be multiple of 16");
        return TID_ERR_PARA;
    }

    return _tid_sym_encdec(1, in_buf, in_buf_len, out_buf, out_buf_len, enc_type, kid);
}


/** 
 * @brief    Symmetrical decryption
 * @param    enc_type                 key type for encryption
 * @param    in_buf                   input buffer
 * @param    in_buf_len               buffer length
 * @param    out_buf                  output buffer
 * @param    out_buf_len              output buffer length pointer
 * @param    kid                      the key index
 * @return                            Operation result
 */
eTidResult tid_sym_dec(uint8_t enc_type, const uint8_t *in_buf, int in_buf_len, 
                        uint8_t *out_buf, uint16_t *out_buf_len, uint8_t kid)
{
    if (0 != (in_buf_len & 0xf)) {
        Log_e("The length of input data must be multiple of 16");
        return TID_ERR_PARA;
    }

    return _tid_sym_encdec(0, in_buf, in_buf_len, out_buf, out_buf_len, enc_type, kid);
}


eTidResult tid_sym_crypto_init(sTidCryptoContext *ctx, uint8_t sym_type, uint8_t enc_algo, uint8_t kid, uint8_t *iv)
{
    _sym_operation.sym_type = sym_type;
    _sym_operation.sym_enc = enc_algo;
    _sym_operation.kid = kid;
    memcpy(_sym_operation.iv, iv, sizeof(_sym_operation.iv));

    ctx->block_id = 0;
    ctx->operation = (void *)&_sym_operation;

    return TID_ERR_SUCCESS;
}

eTidResult tid_sym_crypto_update(sTidCryptoContext *ctx, uint8_t *in_buf, int in_buf_len, uint8_t *out_buf, uint16_t *out_buf_len, int is_last)
{
    eTidResult ret;
    sTidClient *pClient;
    uint16_t sw;
    uint16_t interval = POLL_INTERVAL;
    uint32_t timeout = POLL_TIMEOUT;
    uint8_t  tmp_data[TID_MAX_SEG_LEN + 3];
    int data_index = 0;
    int seg_len;
    uint16_t tmp_out_len;
    int apdu_idx;
    sTidSymOperation *operation = (sTidSymOperation *)ctx->operation;

    pClient = (sTidClient *)get_tid_client_handle();
    if(NULL == pClient)
    {
        Log_e("Tid client is null , pls init tid client first");
        return TID_ERR_FAILURE;
    }

    if (in_buf_len & 0xf) {
        Log_e("The length must be multiple of 16!");
        return TID_ERR_PARA;
    }

    pClient->cmd_case = eAPDU_CASE_D;
    pClient->tid_cmd.cla = TID_CLA_EXT;
    pClient->tid_cmd.ins = TID_INS_SYM_CRYPTO;
    pClient->tid_cmd.p2 = operation->sym_type;
    *out_buf_len = 0;
    
    while (in_buf_len > 0) {
        pClient->tid_cmd.p1 = ctx->block_id;
        apdu_idx = 0;

        if (in_buf_len <= TID_MAX_SEG_LEN) {
            seg_len = in_buf_len;
        } else {
            seg_len = TID_MAX_SEG_LEN;
        }
        if (ctx->block_id == 0) {
            pClient->tid_cmd.apdu[apdu_idx++] = operation->sym_enc;
            pClient->tid_cmd.apdu[apdu_idx++] = operation->kid;
            memset(&(pClient->tid_cmd.apdu[apdu_idx]), 0, TID_SYM_IV_LEN);
            apdu_idx += TID_SYM_IV_LEN;
            memcpy(&(pClient->tid_cmd.apdu[apdu_idx]), &in_buf[data_index], seg_len);
            apdu_idx += seg_len;
            pClient->tid_cmd.lc = apdu_idx;
        } else {
            memcpy(pClient->tid_cmd.apdu, &in_buf[data_index], seg_len);
            pClient->tid_cmd.lc = seg_len;
        }

        if (is_last) {
            ctx->block_id = 0;
            pClient->tid_cmd.p1 |= 0x80;
        }
        
        pClient->tid_cmd.data_len = pClient->tid_cmd.lc;
        pClient->tid_cmd.le = seg_len;
        
        if(_serialize_tid_apdu_packet(pClient) > 0)
        {
            ret = pClient->pfun->tid_apdu_transceive(pClient->send_buf, pClient->send_len, tmp_data,\
                                                                     &tmp_out_len, interval, timeout);
            if(TID_ERR_SUCCESS == ret)
            {
                sw = tmp_data[tmp_out_len - 2]<<8 | tmp_data[tmp_out_len - 1];
#ifdef TID_DEBUG_EABLE		
                Log_d("sw:0x%02x", sw);                 
#endif
                Log_d("tmp_out_len %d seg_len %d", tmp_out_len, seg_len);
                if (TID_SW_SUCC == sw && (tmp_out_len == seg_len + 2)) {
                    memcpy(&out_buf[data_index], tmp_data, tmp_out_len - 2);
                    *out_buf_len += (tmp_out_len - 2);
                } else {
                    Log_e("SymCrypto Error");
                    ret = TID_ERR_FAILURE;
                    break;
                }
            }
            else
            {
                Log_e("SymCrypto failed,ret:%d", ret);
                break;
            }
        }
        else
        {
            Log_e("serialize tid apdu packet failed");
            ret = TID_ERR_FAILURE;  
            break;
        }
        data_index += seg_len;
        ctx->block_id++;
        in_buf_len -= seg_len;
    } 

    return ret;
}


eTidResult tid_asym_crypto_init(sTidCryptoContext *ctx, uint8_t asym_type, uint8_t asym_algo, uint8_t key_source, uint8_t kid, uint8_t *key_buf)
{
    eTidResult ret;
    sTidClient *pClient;
    uint16_t sw;
    uint16_t interval = POLL_INTERVAL;
    uint32_t timeout = POLL_TIMEOUT;
    uint8_t tmp_data[64];
    uint16_t tmp_out_len = 64;
    int apdu_idx = 0;

    pClient = (sTidClient *)get_tid_client_handle();
    if(NULL == pClient)
    {
        Log_e("Tid client is null , pls init tid client first");
        return TID_ERR_FAILURE;
    }

    _asym_operation.asym_type = asym_type;

    ctx->block_id = 0;
    ctx->operation = (void *)&_asym_operation;

    if (key_source == 0x00) {
       return TID_ERR_SUCCESS;
    }

    pClient->cmd_case = eAPDU_CASE_D;
    pClient->tid_cmd.cla = TID_CLA_EXT;
    pClient->tid_cmd.ins = TID_INS_ASYM_CRYPTO;
    pClient->tid_cmd.p2 = asym_type;
    pClient->tid_cmd.p1 = 0x00;

    pClient->tid_cmd.apdu[apdu_idx++] = asym_algo;
    pClient->tid_cmd.apdu[apdu_idx++] = key_source;

    if (key_source) {
        memcpy(&(pClient->tid_cmd.apdu[apdu_idx]), key_buf, TID_PUBKEY_LEN);
        apdu_idx += TID_PUBKEY_LEN;
    } else {
        pClient->tid_cmd.apdu[apdu_idx++] = kid;        
    }
    pClient->tid_cmd.lc = apdu_idx;
    pClient->tid_cmd.data_len = apdu_idx;
    pClient->tid_cmd.le = 0;
    
    if(_serialize_tid_apdu_packet(pClient) > 0)
    {
        ret = pClient->pfun->tid_apdu_transceive(pClient->send_buf, pClient->send_len, tmp_data,\
                                                                 &tmp_out_len, interval, timeout);
        if(TID_ERR_SUCCESS == ret)
        {
            sw = tmp_data[tmp_out_len - 2]<<8 | tmp_data[tmp_out_len - 1];
#ifdef TID_DEBUG_EABLE		
            Log_d("sw:0x%02x", sw);                 
#endif
            if (TID_SW_SUCC != sw) {
                Log_e("ASymCrypto Error");
                return TID_ERR_FAILURE;
            }
        }
        else
        {
            Log_e("ASymCrypto failed,ret:%d", ret);
            return ret;
        }
    }
    else
    {
        Log_e("serialize tid apdu packet failed");
        return TID_ERR_FAILURE;  
    }
    ctx->block_id++;

    return ret;
}

eTidResult tid_asym_crypto_update(sTidCryptoContext *ctx, uint8_t *in_buf, int in_buf_len, uint8_t *out_buf, uint16_t *out_buf_len, int is_last, uint8_t *sign)
{
    eTidResult ret;
    sTidClient *pClient;
    uint16_t sw;
    uint16_t interval = POLL_INTERVAL;
    uint32_t timeout = POLL_TIMEOUT;
    uint8_t  tmp_data[TID_MAX_SEG_LEN];
    int data_index = 0;
    int seg_len;
    uint16_t tmp_out_len;
    int apdu_data_idx = 0;
    sTidAsymOperation *operation = (sTidAsymOperation *)ctx->operation;

    pClient = (sTidClient *)get_tid_client_handle();
    if(NULL == pClient)
    {
        Log_e("Tid client is null , pls init tid client first");
        return TID_ERR_FAILURE;
    }


    pClient->cmd_case = eAPDU_CASE_D;
    pClient->tid_cmd.cla = TID_CLA_EXT;
    pClient->tid_cmd.ins = TID_INS_ASYM_CRYPTO;
    pClient->tid_cmd.p2 = operation->asym_type;
    
    while (in_buf_len > 0) {
        pClient->tid_cmd.p1 = ctx->block_id;

        if (in_buf_len <= TID_MAX_SEG_LEN) {
            seg_len = in_buf_len;
        } else {
            seg_len = TID_MAX_SEG_LEN;
        }

        apdu_data_idx = 0;

        memcpy(&(pClient->tid_cmd.apdu[apdu_data_idx]), &in_buf[data_index], seg_len);
        apdu_data_idx += seg_len;

        if (is_last && operation->asym_type == TID_ASYM_TYPE_SIGN) {
            pClient->tid_cmd.p1 |= 0x80;
            pClient->tid_cmd.le = TID_ASYM_SIGN_LEN;
        } else {
            pClient->tid_cmd.le = 0;
        }
        pClient->tid_cmd.lc = apdu_data_idx;
        pClient->tid_cmd.data_len = pClient->tid_cmd.lc;        
        
        if(_serialize_tid_apdu_packet(pClient) > 0)
        {
            ret = pClient->pfun->tid_apdu_transceive(pClient->send_buf, pClient->send_len, tmp_data,\
                                                                     &tmp_out_len, interval, timeout);
            if(TID_ERR_SUCCESS == ret)
            {
                sw = tmp_data[tmp_out_len - 2]<<8 | tmp_data[tmp_out_len - 1];
#ifdef TID_DEBUG_EABLE		
                Log_d("sw:0x%02x", sw);                 
#endif
                if (TID_SW_SUCC != sw) {
                    Log_e("SymCrypto Error");
                    ret = TID_ERR_FAILURE;
                    break;
                }

                /* output the signature */
                if (is_last && TID_ASYM_TYPE_SIGN == operation->asym_type) {
                    memcpy(out_buf, tmp_data, TID_ASYM_SIGN_LEN);
                    *out_buf_len = TID_ASYM_SIGN_LEN;
                }
            }
            else
            {
                Log_e("SymCrypto failed,ret:%d", ret);
                break;
            }
        }
        else
        {
            Log_e("serialize tid apdu packet failed");
            ret = TID_ERR_FAILURE;  
            break;
        }
        data_index += seg_len;
        ctx->block_id++;
        in_buf_len -= seg_len;
    } 

    /* Append signature data in the last block if it is verification */
    if (ret == TID_ERR_SUCCESS && is_last && TID_ASYM_TYPE_VERIFY == operation->asym_type) {
        pClient->tid_cmd.p1 = ctx->block_id | 0x80;

        memcpy(pClient->tid_cmd.apdu, sign, TID_ASYM_SIGN_LEN);
        pClient->tid_cmd.le = 0;

        pClient->tid_cmd.lc = apdu_data_idx;
        pClient->tid_cmd.data_len = pClient->tid_cmd.lc;        
        
        if(_serialize_tid_apdu_packet(pClient) > 0)
        {
            ret = pClient->pfun->tid_apdu_transceive(pClient->send_buf, pClient->send_len, tmp_data,\
                                                                     &tmp_out_len, interval, timeout);
            if(TID_ERR_SUCCESS == ret)
            {
                sw = tmp_data[tmp_out_len - 2]<<8 | tmp_data[tmp_out_len - 1];
#ifdef TID_DEBUG_EABLE		
                Log_d("sw:0x%02x", sw);                 
#endif
                if (TID_SW_SUCC != sw) {
                    Log_e("SymCrypto Error");
                    return TID_ERR_FAILURE;
                }               
            }
            else
            {
                Log_e("SymCrypto failed,ret:%d", ret);
                return ret;
            }
        }
        else
        {
            Log_e("serialize tid apdu packet failed");
            return TID_ERR_FAILURE;  
        }
    }

    return ret;
}


/** 
 * @brief    asymmetrical sign
 * @param    in_buf                   input buffer
 * @param    in_buf_len               length of input buffer
 * @param    out_buf                  buffer for the signature
 * @param    out_buf_len              the length of the signature
 * @param    kid                      private key index for sign
 * @param    algo                     asymmetrical algorithm
 * @return                            operation result
 */
eTidResult tid_asym_crypto_sign(const uint8_t *in_buf, int in_buf_len, uint8_t *out_buf, uint16_t *out_buf_len, uint8_t kid, uint8_t algo)
{
    eTidResult ret;
    sTidClient *pClient;
    uint16_t sw;
    uint16_t interval = POLL_INTERVAL;
    uint32_t timeout = POLL_TIMEOUT;
    uint8_t  tmp_data[TID_MAX_SEG_LEN + 3];
    int data_index = 0;
    int blk_num = 0;
    int seg_len;
    uint16_t tmp_out_len;
    int is_last = 0;

    pClient = (sTidClient *)get_tid_client_handle();
    if(NULL == pClient)
    {
        Log_e("Tid client is null , pls init tid client first");
        return TID_ERR_FAILURE;
    }

    pClient->cmd_case = eAPDU_CASE_D;
    pClient->tid_cmd.cla = TID_CLA_EXT;
    pClient->tid_cmd.ins = TID_INS_ASYM_CRYPTO;
    pClient->tid_cmd.p2 = TID_ASYM_TYPE_SIGN;
    *out_buf_len = 0;
    pClient->tid_cmd.le = 0x40;
    
    while (in_buf_len > 0) {
        pClient->tid_cmd.p1 = blk_num;

        if (in_buf_len <= TID_MAX_SEG_LEN) {
            pClient->tid_cmd.p1 |= 0x80;
            seg_len = in_buf_len;
            is_last = 1;
        } else {
            seg_len = TID_MAX_SEG_LEN;
        }
        if (blk_num == 0) {
            pClient->tid_cmd.apdu[0] = algo;
            pClient->tid_cmd.apdu[1] = 0x00;
            pClient->tid_cmd.apdu[2] = kid;
            memcpy(&(pClient->tid_cmd.apdu[3]), &in_buf[data_index], seg_len);
            pClient->tid_cmd.lc = seg_len + 3;
        } else {
            memcpy(pClient->tid_cmd.apdu, &in_buf[data_index], seg_len);
            pClient->tid_cmd.lc = seg_len;
        }
        pClient->tid_cmd.data_len = pClient->tid_cmd.lc;
        
        if(_serialize_tid_apdu_packet(pClient) > 0)
        {
            ret = pClient->pfun->tid_apdu_transceive(pClient->send_buf, pClient->send_len, tmp_data,\
                                                                     &tmp_out_len, interval, timeout);
            if(TID_ERR_SUCCESS == ret)
            {
                sw = tmp_data[tmp_out_len - 2]<<8 | tmp_data[tmp_out_len - 1];
#ifdef TID_DEBUG_EABLE		
                Log_d("sw:0x%02x", sw);                 
#endif
                if (TID_SW_SUCC != sw) {
                    Log_e("SymCrypto Error");
                    ret = TID_ERR_FAILURE;
                    break;
                }

                if (is_last) {
                    memcpy(out_buf, tmp_data, tmp_out_len - 2);
                    *out_buf_len = tmp_out_len - 2;
                }                
            }
            else
            {
                Log_e("SymCrypto failed,ret:%d", ret);
                break;
            }
        }
        else
        {
            Log_e("serialize tid apdu packet failed");
            ret = TID_ERR_FAILURE;  
            break;
        }
        data_index += seg_len;
        blk_num++;
        in_buf_len -= seg_len;
    } 

    return ret;
}


eTidResult tid_asym_crypto_verify_inner(const uint8_t *in_buf, int in_buf_len, const uint8_t *sign_buf, uint8_t kid, uint8_t algo)
{
    eTidResult ret;
    sTidClient *pClient;
    uint16_t sw;
    uint16_t interval = POLL_INTERVAL;
    uint32_t timeout = POLL_TIMEOUT;
    uint8_t  tmp_data[TID_MAX_SEG_LEN + 3 + TID_ASYM_SIGN_LEN];
    int data_index = 0;
    int blk_num = 0;
    int seg_len;
    uint16_t tmp_out_len;
    int apdu_data_idx = 0;
    int is_last = 0;

    pClient = (sTidClient *)get_tid_client_handle();
    if(NULL == pClient)
    {
        Log_e("Tid client is null , pls init tid client first");
        return TID_ERR_FAILURE;
    }


    pClient->cmd_case = eAPDU_CASE_C;
    pClient->tid_cmd.cla = TID_CLA_EXT;
    pClient->tid_cmd.ins = TID_INS_ASYM_CRYPTO;
    pClient->tid_cmd.p2 = TID_ASYM_TYPE_VERIFY;
    
    while (in_buf_len > 0) {
        pClient->tid_cmd.p1 = blk_num;

        if (in_buf_len <= TID_MAX_SEG_LEN) {
            pClient->tid_cmd.p1 |= 0x80;
            seg_len = in_buf_len;
            is_last = 1;
        } else {
            seg_len = TID_MAX_SEG_LEN;
        }

        apdu_data_idx = 0;
        if (blk_num == 0) {
            pClient->tid_cmd.apdu[apdu_data_idx++] = algo;
            pClient->tid_cmd.apdu[apdu_data_idx++] = 0x00;
            pClient->tid_cmd.apdu[apdu_data_idx++] = kid;
            
        }
        memcpy(&(pClient->tid_cmd.apdu[apdu_data_idx]), &in_buf[data_index], seg_len);
        apdu_data_idx += seg_len;

        if (is_last) {
            memcpy(&(pClient->tid_cmd.apdu[apdu_data_idx]), sign_buf, TID_ASYM_SIGN_LEN);
            apdu_data_idx += TID_ASYM_SIGN_LEN;
        }
        pClient->tid_cmd.lc = apdu_data_idx;
        
        pClient->tid_cmd.data_len = pClient->tid_cmd.lc;
        
        if(_serialize_tid_apdu_packet(pClient) > 0)
        {
            ret = pClient->pfun->tid_apdu_transceive(pClient->send_buf, pClient->send_len, tmp_data,\
                                                                     &tmp_out_len, interval, timeout);
            if(TID_ERR_SUCCESS == ret)
            {
                sw = tmp_data[tmp_out_len - 2]<<8 | tmp_data[tmp_out_len - 1];
#ifdef TID_DEBUG_EABLE		
                Log_d("sw:0x%02x", sw);                 
#endif
                if (TID_SW_SUCC != sw) {
                    Log_e("SymCrypto Error");
                    ret = TID_ERR_FAILURE;
                    break;
                }               
            }
            else
            {
                Log_e("SymCrypto failed,ret:%d", ret);
                break;
            }
        }
        else
        {
            Log_e("serialize tid apdu packet failed");
            ret = TID_ERR_FAILURE;  
            break;
        }
        data_index += seg_len;
        blk_num++;
        in_buf_len -= seg_len;
    } 

    return ret;
}



eTidResult tid_asym_crypto_verify_outer(const uint8_t *in_buf, int in_buf_len, const uint8_t *sign_buf, const uint8_t *pbkey, uint8_t algo)
{
    eTidResult ret;
    sTidClient *pClient;
    uint16_t sw;
    uint16_t interval = POLL_INTERVAL;
    uint32_t timeout = POLL_TIMEOUT;
    uint8_t  tmp_data[256];
    int data_index = 0;
    int blk_num = 0;
    int seg_len;
    uint16_t tmp_out_len;
    int apdu_data_idx = 0;
    int is_last = 0;
    int max_seg_len;

    pClient = (sTidClient *)get_tid_client_handle();
    if(NULL == pClient)
    {
        Log_e("Tid client is null , pls init tid client first");
        return TID_ERR_FAILURE;
    }


    pClient->cmd_case = eAPDU_CASE_C;
    pClient->tid_cmd.cla = TID_CLA_EXT;
    pClient->tid_cmd.ins = TID_INS_ASYM_CRYPTO;
    pClient->tid_cmd.p2 = TID_ASYM_TYPE_VERIFY;
    
    while (in_buf_len > 0) {
        pClient->tid_cmd.p1 = blk_num;

        if (blk_num == 0) {
            max_seg_len = TID_MAX_SEG_LEN - 64;
        } else {
            max_seg_len = TID_MAX_SEG_LEN;
        }

        if (in_buf_len <= max_seg_len) {
            pClient->tid_cmd.p1 |= 0x80;
            seg_len = in_buf_len;
            is_last = 1;
        } else {
            seg_len = max_seg_len;
        }

        apdu_data_idx = 0;
        if (blk_num == 0) {
            pClient->tid_cmd.apdu[apdu_data_idx++] = algo;
            pClient->tid_cmd.apdu[apdu_data_idx++] = 0x01;
            memcpy(&(pClient->tid_cmd.apdu[apdu_data_idx]), pbkey, TID_PUBKEY_LEN);
            apdu_data_idx += TID_PUBKEY_LEN;
        } 
        memcpy(&(pClient->tid_cmd.apdu[apdu_data_idx]), &in_buf[data_index], seg_len);
        apdu_data_idx += seg_len;

        if (is_last) {
            memcpy(&(pClient->tid_cmd.apdu[apdu_data_idx]), sign_buf, TID_ASYM_SIGN_LEN);
            apdu_data_idx += TID_ASYM_SIGN_LEN;
        }
        pClient->tid_cmd.lc = apdu_data_idx;
        
        pClient->tid_cmd.data_len = pClient->tid_cmd.lc;
        
        if(_serialize_tid_apdu_packet(pClient) > 0)
        {
            ret = pClient->pfun->tid_apdu_transceive(pClient->send_buf, pClient->send_len, tmp_data,\
                                                                     &tmp_out_len, interval, timeout);
            if(TID_ERR_SUCCESS == ret)
            {
                sw = tmp_data[tmp_out_len - 2]<<8 | tmp_data[tmp_out_len - 1];
#ifdef TID_DEBUG_EABLE		
                Log_d("sw:0x%02x", sw);                 
#endif
                if (TID_SW_SUCC != sw) {
                    Log_e("SymCrypto Error");
                    ret = TID_ERR_FAILURE;
                    break;
                }            
            }
            else
            {
                Log_e("AsymCrypto failed,ret:%d", ret);
                break;
            }
        }
        else
        {
            Log_e("serialize tid apdu packet failed");
            ret = TID_ERR_FAILURE;  
            break;
        }
        data_index += seg_len;
        blk_num++;
        in_buf_len -= seg_len;
    } 

    return ret;
}



/** 
 * @brief   Generate temporary key pair
 * @param    pub_key                  The public key of the key pair
 * @param    out_pk_len               The length of public key
 * @param    algo                     The algorithm of the key pair
 * @return                            operation result
 */
eTidResult tid_gen_temp_keypair(uint8_t *pub_key, uint16_t *out_pk_len, uint8_t algo)
{
	eTidResult ret;
    sTidClient *pClient;
    uint16_t sw;
    uint16_t interval = POLL_INTERVAL;
    uint32_t timeout = POLL_TIMEOUT;
    uint8_t  rbuf[128];
    uint16_t rlen;
    
    pClient = (sTidClient *)get_tid_client_handle();
    if(NULL == pClient)
    {
        Log_e("Tid client is null , pls init tid client first");
        return TID_ERR_FAILURE;        
    }
    
    pClient->cmd_case = eAPDU_CASE_B;
    pClient->tid_cmd.cla = TID_CLA_EXT;
    pClient->tid_cmd.ins = TID_INS_GEN_KEYPAIR;
    pClient->tid_cmd.p1 = 0xF0;
    pClient->tid_cmd.p2 = algo;
	pClient->tid_cmd.data_len = 0;
    pClient->tid_cmd.le = TID_PUBKEY_LEN;
    
    if(_serialize_tid_apdu_packet(pClient) > 0)
    {
        ret = pClient->pfun->tid_apdu_transceive(pClient->send_buf, pClient->send_len, rbuf,\
                                                                 &rlen, interval, timeout);
        if(TID_ERR_SUCCESS == ret)
        {
            sw = rbuf[rlen - 2]<<8 | rbuf[rlen - 1];
#ifdef TID_DEBUG_EABLE		
            Log_d("sw:0x%02x", sw);                 
#endif
            if (TID_SW_SUCC != sw) {
                Log_e("Failed, sw is %04x\n", sw);
                ret = TID_ERR_FAILURE;
            } else {
                memcpy(pub_key, rbuf, TID_PUBKEY_LEN);
                *out_pk_len = TID_PUBKEY_LEN;
            }
        }
        else
        {
            Log_e("Get tmp keypair failed,ret:%d", ret);
        }
    }
    else
    {
        Log_e("serialize tid apdu packet failed");
        ret = TID_ERR_FAILURE;  
    }

    return ret;
}

/** 
 * @brief [tid_gen_token description]
 * @param    in_pubkey                [description]
 * @param    in_pubkey_len            [description]
 * @param    algo                     [description]
 * @param    no_return                [description]
 * @param    out_buf                  [description]
 * @return                            [description]
 */
eTidResult tid_gen_token(const uint8_t *in_pubkey, uint16_t in_pubkey_len, uint8_t algo, 
                    int no_return, uint8_t *out_buf)
{
    eTidResult ret;
    sTidClient *pClient;
    uint16_t sw;
    uint16_t interval = POLL_INTERVAL;
    uint32_t timeout = POLL_TIMEOUT;
    int apdu_idx = 0;
    uint8_t tmp_buf[256];
    uint16_t tmp_len;
    
    pClient = (sTidClient *)get_tid_client_handle();
    if(NULL == pClient)
    {
        Log_e("Tid client is null , pls init tid client first");
        return TID_ERR_FAILURE;        
    }

    
    pClient->cmd_case = eAPDU_CASE_D;
    pClient->tid_cmd.cla = TID_CLA_EXT;
    pClient->tid_cmd.ins = TID_INS_GEN_SHAREDKEY;
    pClient->tid_cmd.p1 = algo;
    if (no_return) {
        pClient->tid_cmd.p1 |= 0x80;
    }
    pClient->tid_cmd.p2 = TID_KID_TEMPPRIVKEY;          /* use temp key idx */
    pClient->tid_cmd.lc = in_pubkey_len + 4;             /* public key */
    pClient->tid_cmd.apdu[apdu_idx++] = (algo == 0x01) ? 0x72 : 0x60;
    pClient->tid_cmd.apdu[apdu_idx++] = 0x20;
    memcpy(&pClient->tid_cmd.apdu[apdu_idx], in_pubkey, 0x20);
    apdu_idx += 0x20;
    pClient->tid_cmd.apdu[apdu_idx++] = (algo == 0x01) ? 0x73 : 0x61;
    pClient->tid_cmd.apdu[apdu_idx++] = 0x20;
    memcpy(&pClient->tid_cmd.apdu[apdu_idx], in_pubkey + 0x20, 0x20);
    apdu_idx += 0x20;
    pClient->tid_cmd.data_len = apdu_idx;
    
    if(_serialize_tid_apdu_packet(pClient) > 0)
    {
        ret = pClient->pfun->tid_apdu_transceive(pClient->send_buf, pClient->send_len, tmp_buf,\
                                                                 &tmp_len, interval, timeout);
        if(TID_ERR_SUCCESS == ret)
        {
            sw = tmp_buf[tmp_len - 2]<<8 | tmp_buf[tmp_len - 1];
#ifdef TID_DEBUG_EABLE		
            Log_d("sw:0x%02x %d", sw, tmp_len);
#endif		
            if (TID_SW_SUCC != sw) {
                Log_e("Failed, sw is %04x\n", sw);
                ret = TID_ERR_FAILURE;
            } else if (!no_return) {
                memcpy(out_buf, tmp_buf, TID_PUBKEY_LEN);
            }
        }
        else
        {
            Log_e("Get tmp keypair fail,ret:%d", ret);
        }
    }
    else
    {
        Log_e("serialize tid apdu packet fail");
        ret = TID_ERR_FAILURE;  
    }
        
    return ret;
}


/**
 * @brief tid generate a shared key
 *
 * @param wait bo be define
 *
 * @return @see eTidResult
 */

eTidResult tid_gen_session_key(const uint8_t *prf, uint16_t prf_len)
{
    eTidResult ret;
    sTidClient *pClient;
    uint16_t sw;
    uint16_t interval = POLL_INTERVAL;
    uint32_t timeout = POLL_TIMEOUT;
    uint8_t cmd_out_buf[TID_PUBKEY_LEN + 16];
    uint16_t out_len;
    
    pClient = (sTidClient *)get_tid_client_handle();
    if(NULL == pClient)
    {
        Log_e("Tid client is null , pls init tid client first");
        return TID_ERR_FAILURE;        
    }

    
    pClient->cmd_case = eAPDU_CASE_C;
    pClient->tid_cmd.cla = TID_CLA_EXT;
    pClient->tid_cmd.ins = TID_INS_GEN_SESSIONKEY;
    pClient->tid_cmd.p1 = 0x00;
    pClient->tid_cmd.p2 = 0x41;      /* AES */
    pClient->tid_cmd.lc = prf_len;   /* public key */
    memcpy(pClient->tid_cmd.apdu, prf, prf_len);
	pClient->tid_cmd.data_len = prf_len;
    
    if(_serialize_tid_apdu_packet(pClient) > 0)
    {
        ret = pClient->pfun->tid_apdu_transceive(pClient->send_buf, pClient->send_len, cmd_out_buf,\
                                                                 &out_len, interval, timeout);
        if(TID_ERR_SUCCESS == ret)
        {
            sw = cmd_out_buf[out_len - 2]<<8 | cmd_out_buf[out_len - 1];
	
            if (TID_SW_SUCC != sw) {
                Log_e("Failed, sw is %04x\n", sw);
                ret = TID_ERR_FAILURE;
            }
        }
        else
        {
            Log_e("Failed to generate session key, ret:%d", ret);
        }
    }
    else
    {
        Log_e("serialize tid apdu packet failed");
        ret = TID_ERR_FAILURE;  
    }

    return ret;
}


/**
 * @brief tid compute digest
 *
 * @param wait bo be define
 *
 * @return @see eTidResult
 */

eTidResult tid_compute_digest(int kid, const uint8_t *in_buf, int in_buf_len, 
                        uint8_t *out_buf, uint16_t *out_buf_len, int algo)
{
    eTidResult ret;
    sTidClient *pClient;
    uint16_t sw;
    uint16_t interval = POLL_INTERVAL;
    uint32_t timeout = POLL_TIMEOUT;
    int apdu_idx = 0;
    int data_idx = 0;
    int blk_num = 0;
    int seg_len;
    int is_last = 0;
    uint8_t tmp_buf[128];
    
    pClient = (sTidClient *)get_tid_client_handle();
    if(NULL == pClient)
    {
        Log_e("Tid client is null , pls init tid client first");
        return TID_ERR_FAILURE;        
    }

    /* fix header */
    pClient->cmd_case = eAPDU_CASE_C;
    pClient->tid_cmd.cla = TID_CLA_EXT;
    pClient->tid_cmd.ins = TID_INS_COMPUTE_DIG;
    pClient->tid_cmd.p1 = 0x01;
    pClient->tid_cmd.p2 = algo;  /* data length is less than 255 */

    while(in_buf_len > 0) {
        if (in_buf_len <= TID_MAX_SEG_LEN) {
            is_last = 1;
            seg_len = in_buf_len;
        } else {
            seg_len = TID_MAX_SEG_LEN;
        }
        apdu_idx = 0;

        pClient->tid_cmd.p1 = blk_num;

        if (is_last) {
            pClient->tid_cmd.p1 |= 0x80;
            pClient->cmd_case = eAPDU_CASE_D;
            pClient->tid_cmd.le = 0x20;
        }

        /* Add data header */
        if (blk_num == 0) {
            if (kid >= 0) {
                pClient->tid_cmd.apdu[apdu_idx++] = 0x01;
                pClient->tid_cmd.apdu[apdu_idx++] = kid;
            } else {
                pClient->tid_cmd.apdu[apdu_idx++] = 0x00;
            }
        }
        memcpy(&(pClient->tid_cmd.apdu[apdu_idx]), &in_buf[data_idx], seg_len);
        apdu_idx += seg_len;
        pClient->tid_cmd.lc = apdu_idx;
        pClient->tid_cmd.data_len = apdu_idx;
        
        if(_serialize_tid_apdu_packet(pClient) > 0)
        {
            ret = pClient->pfun->tid_apdu_transceive(pClient->send_buf, pClient->send_len, tmp_buf,\
                                                                     out_buf_len, interval, timeout);
            if(TID_ERR_SUCCESS == ret)
            {
#ifdef TID_DEBUG_EABLE		
                sw = tmp_buf[*out_buf_len - 2]<<8 | tmp_buf[*out_buf_len - 1];
                Log_d("sw:0x%02x", sw);                 
#endif		
                if (TID_SW_SUCC != sw) {
                    ret = TID_ERR_APDU;
                    break;
                }
                if (is_last) {
                    memcpy(out_buf, tmp_buf, *out_buf_len - 2);
                    *out_buf_len -= 2;
                }
            }
            else
            {
                Log_e("Compute digest fail,ret:%d", ret);
                break;
            }
        }
        else
        {
            Log_e("serialize tid apdu packet failed");
            ret = TID_ERR_FAILURE;  
        }
        data_idx += seg_len;
        blk_num++;
        in_buf_len -= seg_len;
    }

    return ret;
}







/**
 * @brief tid storage and read key or custom data
 *
 * @param wait bo be define
 *
 * @return @see eTidResult
 */

eTidResult tid_get_dev_pubkey(uint8_t *out_data, uint16_t *out_data_len)
{
    eTidResult ret;
    sTidClient *pClient;
    uint16_t sw;
    uint16_t interval = POLL_INTERVAL;
    uint32_t timeout = POLL_TIMEOUT;
    uint8_t tmp_data[128];
    
    pClient = (sTidClient *)get_tid_client_handle();
    if(NULL == pClient)
    {
        Log_e("Tid client is null , pls init tid client first");
        return TID_ERR_FAILURE;        
    }
    
    pClient->cmd_case = eAPDU_CASE_B;
    pClient->tid_cmd.cla = TID_CLA_EXT;
    pClient->tid_cmd.ins = TID_INS_SEC_STORE;
    pClient->tid_cmd.p1 = 0x80;      /* Read Data */
    pClient->tid_cmd.p2 = TID_KID_PUBKEY;  
    pClient->tid_cmd.le = TID_PUBKEY_LEN;
 
    if(_serialize_tid_apdu_packet(pClient) > 0)
    {
        ret = pClient->pfun->tid_apdu_transceive(pClient->send_buf, pClient->send_len, tmp_data,\
                                                                 out_data_len, interval, timeout);
        if(TID_ERR_SUCCESS == ret)
        {
            sw = tmp_data[*out_data_len - 2]<<8 | tmp_data[*out_data_len - 1];
#ifdef TID_DEBUG_EABLE		
            Log_d("got sw 0x%02x", sw);   
#endif		
            if (TID_SW_SUCC != sw) {
                Log_e("Failed, sw is %04x\n", sw);
                ret = TID_ERR_FAILURE;
            }
            memcpy(out_data, tmp_data, *out_data_len - 2);
            *out_data_len -= 2;
        }
        else
        {
            Log_e("Security store error, ret:%d", ret);
        }
    }
    else
    {
        Log_e("serialize tid apdu packet fail");
        ret = TID_ERR_FAILURE;  
    }
        
    return ret;

}


/**
 * @brief tid raw apdu input
 *
 * @param apdu          apdu bytes
 * @param len           length of apdu
 * @param output        return bytes of apdu
 * @param outlen        the pointer of output length
 *
 * @return @see eTidResult
 */

eTidResult tid_raw_apdu(uint8_t *apdu, uint16_t len, uint8_t *output, uint16_t *outlen)
{
    eTidResult ret;
    sTidClient *pClient;
    uint16_t sw;
    uint16_t interval = POLL_INTERVAL;
    uint32_t timeout = POLL_TIMEOUT;
    
    pClient = (sTidClient *)get_tid_client_handle();
    if(NULL == pClient)
    {
        Log_e("Tid client is null , pls init tid client first");
        ret = TID_ERR_FAILURE;
        
        goto end;
    }

    ret = pClient->pfun->tid_apdu_transceive(apdu, len, output, outlen, interval, timeout);
    if(TID_ERR_SUCCESS == ret)
    {
        sw = output[*outlen - 2]<<8 | output[*outlen - 1];
#ifdef TID_DEBUG_EABLE		
        Log_d("got sw 0x%02x", sw);   
#endif		
        if (TID_SW_SUCC != sw) {
            Log_e("Failed, sw is %04x\n", sw);
            ret = TID_ERR_FAILURE;
        }
    }
    else
    {
        Log_e("Error, ret:%d", ret);
    }
        
end:
    return ret;

}

