/*
 * Tencent is pleased to support the open source community by making IoT Hub available.
 * Copyright (C) 2019 THL A29 Limited, a Tencent company. All rights reserved.

 * Licensed under the MIT License (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://opensource.org/licenses/MIT

 * Unless required by applicable law or agreed to in writing, software distributed under the License is
 * distributed on an "AS IS" basis, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
/*!
 * @file  tid_client.c
 * @brief implement the authentication of client for tid
 *
 * @details
 *
 * @version 
 *
 * @history
 *
 */
#include "tid_client.h"

#include <string.h>     /* memcpy */

#include "tid_sanity_check.h"
#include "tid_api_inf.h"

#define MAX_SESSION_NUM     8

typedef struct _tid_client_authcode {
    uint8_t     proto_ver;
    uint8_t     suite_id;
    uint8_t     tid_id[TID_ID_LEN];
    uint8_t     nonce[TID_NONCE_LEN];
    uint8_t     client_pubkey[TID_PUBKEY_TOTAL_LEN];
    uint8_t     sign_info[TID_SIGN_LEN];
} __attribute__((packed)) tid_client_code_t;

typedef struct _tid_server_authcode {
    uint8_t     proto_ver;
    uint8_t     suite_id;
    uint8_t     nonce[TID_NONCE_LEN];
    uint8_t     server_pubkey[TID_PUBKEY_TOTAL_LEN];
    uint8_t     sign_info[TID_SIGN_LEN];
} __attribute__((packed)) tid_server_code_t;

typedef struct _tid_suite_type {
    int         suite_id;
    uint8_t     asym_type;
    uint8_t     dh_algo;
    uint8_t     sym_algo;
}tid_suite_type_t;

enum {
    ASYM_SECP256R1  = 0x0A,
    ASYM_SM2        = 0x02,
};

enum {
    DH_SECP256R1    = 0x01,
    DH_SM2          = 0x02,
};

enum {
    DIGEST_SHA256   = 0x02,
    DIGEST_SM3      = 0x05,
};

enum {
    SYM_AES_CBC     = 0x02,
    SYM_SM4_CBC     = 0x10,    
};

enum {
    SYM_SESS_KID_BASE   = 0xF2,
    SYM_SESS_KID_INNER  = 0xFB,
};

enum {
    ASYM_SIGN_ALGO_ECDSA    = 0x06,
    ASYM_SIGN_ALGO_SM2_SM3  = 0x05,
};

static tid_suite_type_t suite_types[] = {
    {
        TID_SUITE_SECP256R1_SM4,
        ASYM_SECP256R1,
        DH_SECP256R1,
        SYM_SM4_CBC
    },
    {
        TID_SUITE_SM2_SM4,
        ASYM_SM2,
        DH_SM2,
        SYM_SM4_CBC
    },
    {
        TID_SUITE_SM2_AES128,
        ASYM_SM2,
        DH_SM2,
        SYM_AES_CBC
    },
    {
        TID_SUITE_SM2_AES256,
        ASYM_SM2,
        DH_SM2,
        SYM_AES_CBC
    },
    {
        TID_SUITE_SECP256R1_SM4,
        ASYM_SECP256R1,
        DH_SECP256R1,
        SYM_SM4_CBC
    },
    {
        TID_SUITE_SECP256R1_AES128,
        ASYM_SECP256R1,
        DH_SECP256R1,
        SYM_AES_CBC
    },
    {
        TID_SUITE_SECP256R1_AES256,
        ASYM_SECP256R1,
        DH_SECP256R1,
        SYM_AES_CBC
    }
};


static sTidClient *g_tid_client = NULL;
static uint8_t _my_sign[TID_SIGN_LEN];
static uint8_t _my_tid[TID_ID_LEN];
static int _tid_init = 0;
static uint8_t _digest_algo;
static tid_suite_type_t *g_suite_type = NULL;

eTidResult tid_client_se_init(eBusType type, eVendor vendor, int suite_type, int is_sm)
{
    eTidResult ret;
    int i;
    uint8_t test_data[256];
    uint16_t rlen;

    if (!g_tid_client) {
        g_tid_client = tid_client_init(type, vendor);

        if (!g_tid_client) {
            Log_e("tid_client_init failed");
            return TID_ERR_INIT;
        }
    }

    g_tid_client->pfun->tid_open_device();   
	g_tid_client->pfun->tid_device_init();  

    ret = g_tid_client->pfun->tid_dev_power_on(test_data, &rlen); 
	if(TID_ERR_SUCCESS != ret)
	{
		Log_e("tid_dev_power_on failed, Ret: %d", ret);
        return TID_ERR_INIT;
	}

    if (is_sm) {
        _digest_algo = DIGEST_SM3;
    } else {
        _digest_algo = DIGEST_SHA256;
    }
    
    for (i = 0; i < sizeof(suite_types) / sizeof(suite_types[0]); ++i) {
        if (suite_types[i].suite_id == suite_type) {
            g_suite_type = &suite_types[i];
            break;
        }
    }

    if (!g_suite_type) {
        Log_e("suite %d is not valid", suite_type);
        return TID_ERR_PARA;
    }    

    return TID_ERR_SUCCESS;
}


eTidResult tid_client_get_authcode(uint8_t *auth_buf, int *buf_len)
{
    if (!g_tid_client) {
        return TID_ERR_INIT;
    }

    if (*buf_len < sizeof(tid_client_code_t)) {
        Log_e("The buffer length for client authcode is too small, %d expected!", sizeof(tid_client_code_t));
        return TID_ERR_PARA;
    }

    tid_client_code_t *authcode = (tid_client_code_t *)auth_buf;
    int datalen = 0;
    uint16_t rlen;
    int i;

    authcode->suite_id = g_suite_type->suite_id;
    if (_digest_algo == DIGEST_SM3) {
        authcode->suite_id |= 0x80;
    }
    datalen++;
    authcode->proto_ver = TID_PROTO_VERSION;
    datalen++;

    if (!g_suite_type) {
        Log_e("proto is not initialized");
        return TID_ERR_PARA;
    }

    if (_tid_init == 0) {
        if (tid_get_tid(_my_tid, &rlen) != TID_ERR_SUCCESS) {
            Log_e("Failed to get TID");
            return TID_ERR_FAILURE;
        }
        _tid_init = 1;
    }
    memcpy(authcode->tid_id, _my_tid, TID_ID_LEN);
    datalen += TID_ID_LEN;
    
    if (tid_get_challenge(TID_NONCE_LEN, authcode->nonce, &rlen) != TID_ERR_SUCCESS) {
        Log_e("Failed to get nonce");
        return TID_ERR_FAILURE;
    }
    datalen += rlen;

    authcode->client_pubkey[0] = 0;
    authcode->client_pubkey[1] = 0;
    if (tid_gen_temp_keypair(&(authcode->client_pubkey[2]), &rlen, g_suite_type->asym_type) != TID_ERR_SUCCESS) {
        Log_e("Failed to generate key pair");
        return TID_ERR_FAILURE;
    }
    datalen += TID_PUBKEY_TOTAL_LEN;

    if (tid_compute_digest(TID_KID_PSK, (const uint8_t *)authcode, sizeof(tid_client_code_t) - TID_SIGN_LEN, 
                    authcode->sign_info, &rlen, _digest_algo) != TID_ERR_SUCCESS) {
        Log_e("Failed to compute digest");
        return TID_ERR_FAILURE;
    }
    datalen += TID_SIGN_LEN;
    memcpy(_my_sign, authcode->sign_info, TID_SIGN_LEN);

    *buf_len = datalen;

    return TID_ERR_SUCCESS;
}

eTidResult tid_client_validate_authcode(uint8_t *auth_buf, int buf_len, uint8_t *token, uint16_t *rlen)
{
    uint16_t outlen;
    uint8_t tmp_data[64];

    if (!g_tid_client) {
        return TID_ERR_INIT;
    }
    
    if (!g_suite_type) {
        Log_e("proto is not initialized");
        return TID_ERR_PARA;
    }

    if (buf_len < sizeof(tid_server_code_t)) {
        Log_e("The buffer length for client authcode is too small, %d expected!", sizeof(tid_server_code_t));        return TID_ERR_PARA;
    }

    if (!g_suite_type) {
        Log_e("suite_type is not initialized");
        return TID_ERR_INIT;
    }

    tid_server_code_t *authcode = (tid_server_code_t *)auth_buf;
    int no_return = token == NULL ? 1 : 0;

    /* check version */
    if (authcode->suite_id != g_suite_type->suite_id) {
        Log_e("suite id mismatched");
        return TID_ERR_FAILURE;
    }

    if (authcode->proto_ver != TID_PROTO_VERSION) {
        Log_e("proto version mismatched");
        return TID_ERR_FAILURE;
    }

    /* check the sign */
    uint8_t sign_buf[160];
    uint8_t calc_sign[32];
    uint8_t *p = sign_buf;
    memcpy(p, _my_tid, TID_ID_LEN);
    p += TID_ID_LEN;
    memcpy(p, _my_sign, TID_SIGN_LEN);
    p += TID_SIGN_LEN;
    memcpy(p, authcode->nonce, TID_NONCE_LEN);
    p += TID_NONCE_LEN;
    memcpy(p, authcode->server_pubkey, TID_PUBKEY_TOTAL_LEN);
    p += TID_PUBKEY_TOTAL_LEN;

    if (tid_compute_digest(TID_KID_PSK, sign_buf, p - sign_buf, 
                calc_sign, &outlen, _digest_algo) != TID_ERR_SUCCESS) {
        Log_e("Failed to compute digest");
        return TID_ERR_FAILURE;
    }

    if (memcpy(calc_sign, authcode->sign_info, TID_SIGN_LEN)) {
        Log_e("Sign mismatched");
        return TID_ERR_FAILURE;
    }

    if (tid_gen_token(&(authcode->server_pubkey[2]), TID_PUBKEY_LEN, 
        g_suite_type->dh_algo, no_return, tmp_data) != TID_ERR_SUCCESS) {
        Log_e("Failed to generate sharedkey");
        return TID_ERR_FAILURE;
    }

    memcpy(token, tmp_data, 32);
    *rlen = 32;

    return TID_ERR_SUCCESS;
}

eTidResult tid_client_generate_sesskey(uint8_t *prf, int prf_len)
{
    if (!g_tid_client) {
        Log_e("TID driver is not initialized!");
        return TID_ERR_INIT;
    }

    if (TID_ERR_SUCCESS != tid_gen_session_key(prf, prf_len)) {
        Log_e("Generate session key failed");
        return TID_ERR_FAILURE;
    }

    return TID_ERR_SUCCESS;
}

eTidResult tid_client_import_sesskey(uint8_t *sess_key, int key_len, int key_index)
{
    if (!g_tid_client) {
        Log_e("TID driver is not initialized!");
        return TID_ERR_INIT;
    }
    
    if (!g_suite_type) {
        Log_e("proto is not initialized");
        return TID_ERR_PARA;
    }    

    if (TID_ERR_SUCCESS != tid_import_sesskey(sess_key, key_len, g_suite_type->sym_algo, key_index + SYM_SESS_KID_BASE)) {
        Log_e("TID import session key failed!");
        return TID_ERR_FAILURE;
    }

    return TID_ERR_SUCCESS;
}


eTidResult tid_client_sym_crypto(const uint8_t *in_buf, uint8_t *out_buf, int in_buf_len, int key_index, int is_enc)
{
    uint8_t sym_kid;
    uint16_t rlen;
    eTidResult ret;

    if (!g_tid_client) {
        Log_e("TID driver is not initialized!");
        return TID_ERR_INIT;
    }
    
    if (!g_suite_type) {
        Log_e("proto is not initialized");
        return TID_ERR_PARA;
    }

    if (0 != (in_buf_len & 0xf)) {
        Log_e("The length of buffer must be multiple of 16!");
        return TID_ERR_PARA;
    }
    
    if (key_index < 0) {
        sym_kid = SYM_SESS_KID_INNER;
    } else if (key_index >= MAX_SESSION_NUM) {
        Log_e("The key index is out of range (from 0 to %d)!", MAX_SESSION_NUM - 1);
        return TID_ERR_PARA;
    } else {
        sym_kid = SYM_SESS_KID_BASE + key_index;
    }

    if (is_enc) {
        ret = tid_sym_enc(g_suite_type->sym_algo, in_buf, in_buf_len, out_buf, &rlen, sym_kid);

        if (ret != TID_ERR_SUCCESS || rlen != in_buf_len) {
            Log_e("Encryption failed");
            return TID_ERR_FAILURE;
        }
    } else {
        ret = tid_sym_dec(g_suite_type->sym_algo, in_buf, in_buf_len, out_buf, &rlen, sym_kid);
        
        if (ret != TID_ERR_SUCCESS || rlen != in_buf_len) {
            Log_e("Encryption failed");
            return TID_ERR_FAILURE;
        }
    }

    return ret;
}

eTidResult tid_client_digest(const uint8_t *in_buf, int in_buf_len, uint8_t *out_buf)
{
    uint16_t rlen;

    if (!g_tid_client) {
        Log_e("TID driver is not initialized!");
        return TID_ERR_INIT;
    }
    
    return tid_compute_digest(-1, in_buf, in_buf_len, out_buf, &rlen, _digest_algo);
}

eTidResult tid_client_asym_sign(const uint8_t *in_buf, int in_buf_len, uint8_t *out_buf, int *out_buf_len)
{
    if (!g_tid_client) {
        Log_e("TID driver is not initialized!");
        return TID_ERR_INIT;
    }
    
    if (!g_suite_type) {
        Log_e("proto is not initialized");
        return TID_ERR_PARA;
    }

    uint16_t rlen;
    eTidResult ret;
    uint8_t asym_sign_algo = g_suite_type->asym_type == ASYM_SECP256R1 ? ASYM_SIGN_ALGO_ECDSA : ASYM_SIGN_ALGO_SM2_SM3;

    ret = tid_asym_crypto_sign(in_buf, in_buf_len, out_buf, &rlen, TID_KID_PRIVKEY, g_suite_type->asym_type);
    if (TID_ERR_SUCCESS != ret) {
        return ret;
    }

    *out_buf_len = rlen;

    return ret;
}

eTidResult tid_client_asym_verify(const uint8_t *in_msg, int in_msg_len, const uint8_t *in_pubkey, const uint8_t *in_sign_buf)
{
    uint8_t asym_sign_algo = g_suite_type->asym_type == ASYM_SECP256R1 ? ASYM_SIGN_ALGO_ECDSA : ASYM_SIGN_ALGO_SM2_SM3;

    return tid_asym_crypto_verify_outer(in_msg, in_msg_len, in_sign_buf, in_pubkey, asym_sign_algo);
}


