#include "tid_export.h"

#if ((TID_SE_VENDOR == TID_SE_VENDOR_FM) && (TID_BUS_TYPE == TID_BUS_I2C))

#include <stdint.h>
#include "port_i2c.h"
#include "fm_driver_i2c.h"



//variable of I2C SE addr
static uint8_t gfm_I2CAddr;

//struct point of tid I2C driver
static sTid_I2C_Driver *pgfm_I2CDrv = NULL;


static eTidResult fm_i2c_transfer_result(uint8_t ret)
{
	eTidResult resault;
	
	switch(ret)
	{
		case 0:
			resault = TID_ERR_SUCCESS;
			break;
	
		case 1:
			resault = TID_ERR_CRC;
			break;
		case 2:
			resault = TID_ERR_INS;
			break;
		case 11:
			resault = TID_ERR_NULL;
			break;
		case 12:
			resault = TID_ERR_RECV_ACK;
			break;
		case 13:
			resault = TID_ERR_LENGTH;
			break;
		case 14:
			resault = TID_ERR_LRC;
			break;
		default:
			resault = TID_ERR_OTHER;
			break;
	}

	return resault;
}


/********************************************************************
* Function: register I2C driver
* Input Parameter:
* fm_i2c_drv: the struct point of I2C driver
* Output Parameter:None
* Return: None
*********************************************************************/
void fm_i2c_drv_reg(sTid_I2C_Driver *fm_i2c_drv)
{
	pgfm_I2CDrv = fm_i2c_drv;
}

/********************************************************************
* Function: unregister I2C driver
* Input Parameter:None
* Output Parameter:None
* Return: None
*********************************************************************/
void fm_i2c_drv_unreg(void)
{
	pgfm_I2CDrv = NULL;
}

/********************************************************************
* Function: I2C device init
* Input Parameter:None
* Output Parameter:None
* Return: None
*********************************************************************/
void fm_i2c_dev_init(void)
{
	if(pgfm_I2CDrv)
		pgfm_I2CDrv->tid_i2c_init();
}

/********************************************************************
* Function: open I2C device
* Input Parameter:None
* Output Parameter:None
* Return: None
*********************************************************************/
void fm_i2c_open_device(void)
{
	if(pgfm_I2CDrv)
	{
		gfm_I2CAddr = pgfm_I2CDrv->se_i2c_addr;
		pgfm_I2CDrv->tid_i2c_power_on();
	}
}

/********************************************************************
* Function: close I2C device
* Input Parameter:None
* Output Parameter:None
* Return: None
*********************************************************************/
void fm_i2c_close_device(void)
{
	if(pgfm_I2CDrv)
	{
		gfm_I2CAddr = 0;
		pgfm_I2CDrv->tid_i2c_power_off();
	}
}

/********************************************************************
* Function: send a frame to I2C SE
* Input Parameter:
* fm_i2c_drv: the struct point of I2C driver
* sbuf: send frame data
* slen: send frame length
* Output Parameter:None
* Return: result 
* 0-success
* 11-null point err
* 12-wait ack timeout
* other-RFU
*********************************************************************/
eTidResult fm_i2c_send_frame(uint8_t cmd, uint8_t *sbuf, uint16_t slen)
{
	FM_I2C_HEAD fm_i2c_hd;
	uint16_t i;
	uint8_t bcc;
	eTidResult ret;
	
	fm_i2c_hd.lenlo = slen+3;
	fm_i2c_hd.lenhi = (slen+3)>>8;
	fm_i2c_hd.nad = 0;
	fm_i2c_hd.flag.cmd = cmd;

	if(!pgfm_I2CDrv)
	{
		Log_e("i2c driver should init first");
		return TID_ERR_NULL;
	}
	
	pgfm_I2CDrv->tid_i2c_start();

	//send se addr,write slaver
	ret = pgfm_I2CDrv->tid_i2c_send_addr(gfm_I2CAddr);
	if(TID_ERR_SUCCESS != ret) 
	{
		Log_e("send addr err");
		goto END;
	}

	//send pack head
	for(i=0; i<4; i++)
	{
		ret = pgfm_I2CDrv->tid_i2c_send_char(*(&fm_i2c_hd.lenlo+i));
		if(TID_ERR_SUCCESS != ret) 
		{
			Log_e("send head err");
			goto END;
		}
	}

	//calc bcc
	bcc = fm_i2c_hd.lenlo ^ fm_i2c_hd.lenhi ^ fm_i2c_hd.nad ^ fm_i2c_hd.flag.cmd; 

	//send data
	for(i=0; i<slen; i++)
	{
		ret = pgfm_I2CDrv->tid_i2c_send_char(*(sbuf+i));
		bcc ^= sbuf[i];
		if(TID_ERR_SUCCESS != ret) 
		{
			Log_e("send data err");
			goto END;
		}			
	}

	//send bcc
	ret = pgfm_I2CDrv->tid_i2c_send_char(bcc);
	if(TID_ERR_SUCCESS != ret) 
	{
		Log_e("send bcc err");
	}

	pgfm_I2CDrv->tid_i2c_stop();

END:	
	return ret;
}

/********************************************************************
* Function: receive a frame from I2C SE
* Input Parameter:None
* Output Parameter:
* rbuf: received frame data
* rlen: received frame length
* Return: result 
* 0-success
* 1-SE CRC err
* 2-SE INS err
* 0xF2-SE WTX err
* 11-null point err
* 12-receive ack err
* 13-received length err
* 14-received bcc err
* other-RFU
*********************************************************************/
eTidResult fm_i2c_recv_frame(uint8_t *rbuf, uint16_t *rlen)
{
	uint16_t i;
	uint16_t recvLen;
	uint8_t bcc;
	eTidResult ret;
	FM_I2C_HEAD fm_i2c_hd;

	*rlen = 0;
	
	if(!pgfm_I2CDrv)
	{
		Log_e("i2c driver should init first");
		return TID_ERR_NULL;
	}
		
	
	pgfm_I2CDrv->tid_i2c_start();
	
	//send se addr,read slaver
	ret = pgfm_I2CDrv->tid_i2c_send_addr(gfm_I2CAddr+1);
	if(TID_ERR_SUCCESS != ret)
	{
		Log_e("send addr err");
		return ret;
	}

	//recv length
	fm_i2c_hd.lenlo = pgfm_I2CDrv->tid_i2c_recv_char();
	fm_i2c_hd.lenhi = pgfm_I2CDrv->tid_i2c_recv_char();

	recvLen  = (fm_i2c_hd.lenhi << 8) + fm_i2c_hd.lenlo;
	
	if(recvLen < I2C_MIN_LEN || recvLen > I2C_MAX_LEN)
	{
		*rlen = 0;
		return TID_ERR_LENGTH;
	}

	//recv nad and sta
	fm_i2c_hd.nad = pgfm_I2CDrv->tid_i2c_recv_char();
	fm_i2c_hd.flag.sta = pgfm_I2CDrv->tid_i2c_recv_char();

	bcc = fm_i2c_hd.lenlo ^ fm_i2c_hd.lenhi ^ fm_i2c_hd.nad ^ fm_i2c_hd.flag.sta;

	//recv data length
	*rlen = recvLen - 3;
	
	//recv data
	for(i=0; i<*rlen; i++)
	{
		rbuf[i] = pgfm_I2CDrv->tid_i2c_recv_char();
		bcc ^= rbuf[i];
	}
	
	//stop
	pgfm_I2CDrv->tid_i2c_stop();

	//recv and check bcc
	bcc ^= pgfm_I2CDrv->tid_i2c_recv_char();
	
	if(bcc)
	{
		*rlen = 0;
		return TID_ERR_LRC;
	}

	if(0x0 == fm_i2c_hd.flag.sta)
	{
		ret = TID_ERR_SUCCESS;
	}
	else if(0x1 == fm_i2c_hd.flag.sta)
	{
		ret = TID_ERR_CRC;
	}
	else if(0xF2 == fm_i2c_hd.flag.sta)
	{
		ret = TID_ERR_DATA_NOT_READY;
	}
	else
	{
		ret = TID_ERR_FAILURE;
	}

	return ret;
}

/********************************************************************
* Function: get ATR from I2C SE
* Input Parameter:None
* Output Parameter:
* rbuf: received ATR data
* rlen: received ATR length
* Return: result 
* 0-success
* 1-SE CRC err
* 2-SE INS err
* 0xF2-SE WTX err
* 11-null point err
* 12-receive ack err
* 13-received length err
* 14-received bcc err
* other-RFU
*********************************************************************/
eTidResult fm_i2c_get_atr(uint8_t *rbuf, uint16_t *rlen)
{
	eTidResult ret;
	Timer timer;

	TID_FUNC_ENTRY

	ret = fm_i2c_send_frame(I2C_CMD_GET_ATR, 0, 0);
	if(TID_ERR_SUCCESS != ret) 
	{
		Log_e("fm_i2c_send_frame err,%d",ret);
		return ret;
	}	

	HAL_Timer_countdown_ms(&timer, POLL_TIMEOUT);
	do{
		ret = fm_i2c_recv_frame(rbuf, rlen);
		if(TID_ERR_SUCCESS == ret)
		{
			break;
		} 
	}while(!HAL_Timer_expired(&timer));

	return ret;
}

/********************************************************************
* Function: transcieve a frame to I2C SE
* Input Parameter:
* sbuf: send frame data
* slen: send frame length
* timeout: timeout for wait receive a frame
* Output Parameter:
* rbuf: received frame data
* rlen: received frame length
* Return: result 
* 0-success
* 1-SE CRC err
* 2-SE INS err
* 0xF2-SE WTX err
* 11-null point err
* 12-receive ack err
* 13-received length err
* 14-received bcc err
* other-RFU
*********************************************************************/
eTidResult fm_i2c_transceive(uint8_t *sbuf, uint16_t slen, uint8_t *rbuf, uint16_t *rlen, 
						uint16_t poll_inv, uint32_t poll_timeout)
{	
	uint8_t retry_cnt = I2C_RETRY_TIMES;
	uint8_t request_cnt = I2C_REQUEST_TIMES;
	Timer timer;
	eTidResult ret;

RETRY:
	*rlen = 0;
	ret = fm_i2c_send_frame(I2C_CMD_IBLOCK, sbuf, slen);

	if(TID_ERR_SUCCESS != ret)
	{
		return TID_ERR_FAILURE;
	}	

	HAL_Timer_countdown_ms(&timer, poll_timeout);
	do{
		ret = fm_i2c_recv_frame(rbuf, rlen);

		if(ret == TID_ERR_RECV_ACK || ret == TID_ERR_DATA_NOT_READY)
		{	//timeout
			HAL_DelayMs(poll_inv);
		}
		else if(ret == TID_ERR_LENGTH || ret == TID_ERR_LRC)
		{	//receive err
			if(!request_cnt)
			{
				break;
			}				
			request_cnt--;
			fm_i2c_send_frame(I2C_CMD_NAK, 0, 0);	//request SE resend
		}
		else
		{
			break;
		}
			
	}while(!HAL_Timer_expired(&timer));
	
	if(ret == TID_ERR_CRC && retry_cnt)	//SE CRC err
	{
		retry_cnt--;
		goto RETRY;
	}

	return	ret;
}

//global variable of I2C SE 
sTidFunc gfm_func_i2c = 
{
	eBUS_I2C,
	fm_i2c_drv_reg,
	fm_i2c_dev_init,
	fm_i2c_open_device,
	fm_i2c_close_device,
	fm_i2c_get_atr,
	fm_i2c_transceive,
	fm_i2c_drv_unreg
};

#endif
