#include "tid_export.h"
#if ((TID_SE_VENDOR == TID_SE_VENDOR_FM) && (TID_BUS_TYPE == TID_BUS_SPI))

#include <stdint.h>
#include "port_spi.h"
#include "fm_driver_spi.h"



//struct point of SPI SE driver
static sTid_SPI_Driver *pgfm_SPIDrv = NULL;


static eTidResult fm_spi_transfer_result(uint8_t ret)
{
	eTidResult result;
	
	switch(ret)
	{
		case 0:
			result = TID_ERR_SUCCESS;
			break;
		case 1:
			result = TID_ERR_CRC;
			break;
		case 2:
			result = TID_ERR_INS;
			break;
		case 11:
			result = TID_ERR_NULL;
			break;
		case 12:
			result = TID_ERR_RECV_ACK;
			break;
		case 13:
			result = TID_ERR_LENGTH;
			break;
		case 14:
			result = TID_ERR_LRC;
			break;
		default:
			result = TID_ERR_OTHER;
			break;
	}

	return result;
}



/********************************************************************
* Function: register SPI driver
* Input Parameter:
* fm_spi_drv: the struct point of SPI driver
* Output Parameter:None
* Return: None
*********************************************************************/
void fm_spi_drv_reg(sTid_SPI_Driver *fm_spi_drv)
{
	pgfm_SPIDrv = fm_spi_drv;
}

/********************************************************************
* Function: unregister SPI driver
* Input Parameter:None
* Output Parameter:None
* Return: None
*********************************************************************/
void fm_spi_drv_unreg(void)
{
	pgfm_SPIDrv = NULL;
}

/********************************************************************
* Function: SPI device init
* Input Parameter:None
* Output Parameter:None
* Return: None
*********************************************************************/
void fm_spi_dev_init(void)
{
	if(pgfm_SPIDrv)
		pgfm_SPIDrv->tid_spi_init();
}

/********************************************************************
* Function: open SPI device
* Input Parameter:None
* Output Parameter:None
* Return: None
*********************************************************************/
void fm_spi_open_device(void)
{
	if(pgfm_SPIDrv)
		pgfm_SPIDrv->tid_spi_power_on();
}

/********************************************************************
* Function: close SPI device
* Input Parameter:None
* Output Parameter:None
* Return: None
*********************************************************************/
void fm_spi_close_device(void)
{
	if(pgfm_SPIDrv)
		pgfm_SPIDrv->tid_spi_power_off();
}

/********************************************************************
* Function: check SPI SE state
* Input Parameter:
* count: the count of check SPI SE state
* Output Parameter:None
* Return: result 
* 0-ready
* 1-busy
* other-RFU
*********************************************************************/
uint8_t fm_spi_chk_state(uint16_t inv, uint32_t timeout)
{
	uint8_t state = 1;
	Timer timer;

	HAL_Timer_countdown_ms(&timer, timeout);
	//chk state and wait se ok
	do{
		pgfm_SPIDrv->tid_spi_select();
		pgfm_SPIDrv->tid_spi_transceive_char(SPI_CMD_CHK_STATE);
		HAL_DelayUs(SPI_CMD_DELAY);
		state = pgfm_SPIDrv->tid_spi_transceive_char(0xFF);
		pgfm_SPIDrv->tid_spi_deselect();
		if(state)
		{
			HAL_DelayMs(inv);
		}	
		else
		{
			break;
		} 
	}while(!HAL_Timer_expired(&timer));


	Log_d("sta=0x%x",state);

	return state;
}

/********************************************************************
* Function: get device ID from SPI SE
* Input Parameter:None
* Output Parameter:
* devid: received device ID data
* rlen: received device ID length
* Return: result 
* 0-success
* 11-null point err
* other-RFU
*********************************************************************/
eTidResult fm_spi_get_devid(uint8_t *devid, uint16_t *rlen)
{
	uint8_t i;
	uint8_t devid_len = 7;

	*rlen = 0;
	
	if(!pgfm_SPIDrv)
	{
		return fm_spi_transfer_result(11);
	}
		
	pgfm_SPIDrv->tid_spi_select();

	//send spi cmd header
	pgfm_SPIDrv->tid_spi_transceive_char(SPI_CMD_GET_ID);
	HAL_DelayUs(SPI_CMD_DELAY);

	for(i=0; i<devid_len; i++)
	{
		devid[i] = pgfm_SPIDrv->tid_spi_transceive_char(0xFF);
	}

	pgfm_SPIDrv->tid_spi_deselect();
	
	*rlen = devid_len;

	return fm_spi_transfer_result(0);;
}

/********************************************************************
* Function: transcieve a frame to SPI SE
* Input Parameter:
* sbuf: send frame data
* slen: send frame length
* timeout: timeout for wait receive a frame
* Output Parameter:
* rbuf: received frame data
* rlen: received frame length
* Return: result 
* 0-success
* 11-null point err
* 12-wait ack timeout
* 13-received length err
* 14-received bcc err
* other-RFU
*********************************************************************/
eTidResult fm_spi_transceive(uint8_t *sbuf, uint16_t slen, uint8_t *rbuf, uint16_t *rlen, 
						uint16_t poll_inv, uint32_t poll_timeout)
{
	uint8_t LenHi;
	uint8_t LenLo;
	uint8_t Lrc = 0;
	uint16_t i;

#ifdef SPI_DEBUG_EABLE
	HexDump(NULL, sbuf, slen);
#endif
	
	*rlen = 0;	
	if(!pgfm_SPIDrv)
	{
		return fm_spi_transfer_result(11);
	}
		
	
	pgfm_SPIDrv->tid_spi_select();
	
	//send spi cmd header
	pgfm_SPIDrv->tid_spi_transceive_char(SPI_CMD_SEND_DATA);
	HAL_DelayUs(SPI_CMD_DELAY);

	//send length
	LenHi = slen>>8;
	LenLo = slen;
	pgfm_SPIDrv->tid_spi_transceive_char(LenHi);
	pgfm_SPIDrv->tid_spi_transceive_char(LenLo);

	//calc lrc
	Lrc = 0xFF^LenHi^LenLo;
	
	//send frame
	for(i=0;i<slen;i++)
	{
		pgfm_SPIDrv->tid_spi_transceive_char(sbuf[i]);
		Lrc ^= sbuf[i];
	}

	//send lrc
	pgfm_SPIDrv->tid_spi_transceive_char(Lrc);
	
	pgfm_SPIDrv->tid_spi_deselect();

	//debug
	Log_d("HEAD:%02x,%02x,LRC:%02x\r\n",LenHi,LenLo,Lrc);

	
	//chk se state
	if(fm_spi_chk_state(poll_inv, poll_timeout))
	{
		Log_e("spi check state err");
		return fm_spi_transfer_result(12);	
	}
		
	
	pgfm_SPIDrv->tid_spi_select();
	//send spi receive cmd header
	pgfm_SPIDrv->tid_spi_transceive_char(SPI_CMD_RECV_DATA);
	HAL_DelayUs(SPI_CMD_DELAY);

	//read 2bytes length
	LenHi = pgfm_SPIDrv->tid_spi_transceive_char(0xFF);
	LenLo = pgfm_SPIDrv->tid_spi_transceive_char(0xFF);
	*rlen = LenHi<<8 | LenLo;

	if(*rlen < SPI_MIN_LEN || *rlen > SPI_MAX_LEN)
	{
		Log_e("read len err,%d", *rlen);
		*rlen = 0;
		pgfm_SPIDrv->tid_spi_deselect();		
		return fm_spi_transfer_result(13);
	}

	//calc lrc
	Lrc = 0xFF^LenHi^LenLo;
	
	//receive frame
	for(i=0; i<*rlen; i++)
	{
		rbuf[i] = pgfm_SPIDrv->tid_spi_transceive_char(0xFF);
		Lrc ^= rbuf[i];
	}

	//receive lrc byte
	Lrc ^= pgfm_SPIDrv->tid_spi_transceive_char(0xFF);

	pgfm_SPIDrv->tid_spi_deselect();

	if(Lrc)
	{
		Log_e("Lrc err,%d", Lrc);
		*rlen = 0;
		return fm_spi_transfer_result(14);
	}

	return fm_spi_transfer_result(0);
}

//global variable of SPI SE 
sTidFunc gfm_func_spi = 
{
	eBUS_SPI,
	fm_spi_drv_reg,
	fm_spi_dev_init,
	fm_spi_open_device,
	fm_spi_close_device,
	fm_spi_get_devid,
	fm_spi_transceive,
	fm_spi_drv_unreg
};

#endif